@@ -1,14 +1,13 @@
 
 /*
- *  autogen.h
- *  $Id: 851456692b738b0ee9c943bbba6f5c2b9fd7fe43 $
+ *  \file autogen.h
  *
- *  Time-stamp:        "2009-07-09 18:56:32 bkorb"
+ *  Time-stamp:        "2011-04-20 14:25:17 bkorb"
  *
  *  Global header file for AutoGen
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -26,12 +25,16 @@
 #ifndef AUTOGEN_BUILD
 #define AUTOGEN_BUILD 1
 
-#include "config.h"
 #include "compat/compat.h"
+#include "compat/unlocked-io.h"
 
 #include REGEX_HEADER
 #include <libguile/scmconfig.h>
-#include <guile/gh.h>
+#if GUILE_VERSION >= 109000
+# include <libguile.h>
+#else
+# include <guile/gh.h>
+#endif
 
 #include "opts.h"
 #include "expr.h"
@@ -39,17 +42,21 @@
 #include "directive.h"
 #include "snprintfv/printf.h"
 
-#ifndef STR
-#  define _STR(s) #s
-#  define STR(s)  _STR(s)
+#define  LOG10_2to32  10  /* rounded up */
+
+#if defined(SHELL_ENABLED) || defined(DAEMON_ENABLED)
+#  ifndef HAVE_WORKING_FORK
+#    error SHELL is enabled and fork() does not work
+     choke me
+#  endif
 #endif
 
-#ifdef DEFINING
-#  define VALUE( s )  = s
-#  define MODE
-#else
-#  define VALUE( s )
-#  define MODE extern
+#ifndef DIRCH
+# if defined(_WIN32) && !defined(__CYGWIN__)
+#  define DIRCH                  '\\'
+# else
+#  define DIRCH                  '/'
+# endif
 #endif
 
 #define YYSTYPE t_word
@@ -71,10 +78,10 @@ typedef struct {
 
 #define NOPROCESS   ((pid_t)-1)
 #define NULLPROCESS ((pid_t)0)
+#define NL          '\n'
+#define TAB         '\t'
 
-typedef unsigned char* tpChar;
-
-#define STRSIZE( s )  (sizeof(s)-1)
+typedef unsigned char * tpChar;
 
 #include "cgi-fsm.h"
 #include "defParse-fsm.h"
@@ -103,7 +110,7 @@ typedef enum { STATE_TABLE COUNT_PROC_STATE } teProcState;
 
 #define EXPORT
 
-typedef struct fpStack       tFpStack;
+typedef struct fp_stack      tFpStack;
 typedef struct outSpec       tOutSpec;
 typedef struct scanContext   tScanCtx;
 typedef struct defEntry      tDefEntry;
@@ -177,29 +184,29 @@ struct template_lib_marker {
 #define EMIT_NO_DEFINE      0x0800  /* don't get defined value */
 
 struct macro_desc {
-    teFuncType  funcCode;  /* Macro function         */
-    int         lineNo;    /* of macro def           */
-    int         endIndex;  /* End of block macro     */
-    int         sibIndex;  /* Sibling macro (ELIF or SELECT) */
+    teFuncType    funcCode;    /* Macro function           */
+    int           lineNo;      /* of macro def             */
+    int           endIndex;    /* End of block macro       */
+    int           sibIndex;    /* Sibling macro (ELIF or SELECT) */
 
-    uintptr_t   ozName;    /* macro name (sometimes) */
-    uintptr_t   ozText;    /* associated text        */
-    long        res;       /* some sort of result    */
-    void*       funcPrivate;
+    uintptr_t     ozName;      /* macro name (sometimes)   */
+    uintptr_t     ozText;      /* associated text          */
+    uintptr_t     res;         /* some sort of result      */
+    void *        funcPrivate;
 };
 
 struct template_desc {
-    tTlibMark   magic;       /* TEMPLATE_MAGIC_MARKER    */
-    size_t      descSize;    /* Structure Size           */
-    char*       pNext;       /* Next Pointer             */
-    int         macroCt;     /* Count of Macros          */
-    tCC*        pzTplFile;   /* Name of template file    */
-    char*       pzTplName;   /* Defined Macro Name       */
-    char*       pzTemplText; /* offset of the text       */
-    char        zStartMac[MAX_SUFFIX_LEN];
-    char        zEndMac[MAX_SUFFIX_LEN];
-    tMacro      aMacros[1];  /* Array of Macros          */
-/*  char        text[...];    * strings at end of macros */
+    tTlibMark     magic;       /* TEMPLATE_MAGIC_MARKER    */
+    size_t        descSize;    /* Structure Size           */
+    char *        pNext;       /* Next Pointer             */
+    int           macroCt;     /* Count of Macros          */
+    char const *  pzTplFile;   /* Name of template file    */
+    char *        pzTplName;   /* Defined Macro Name       */
+    char *        pzTemplText; /* offset of the text       */
+    char          zStartMac[MAX_SUFFIX_LEN];
+    char          zEndMac[MAX_SUFFIX_LEN];
+    tMacro        aMacros[1];  /* Array of Macros          */
+/*  char          text[...];    * strings at end of macros */
 };
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
@@ -240,11 +247,11 @@ struct defEntry {
 };
 
 struct scanContext {
-    tScanCtx*   pCtx;
-    char*       pzScan;
-    tCC*        pzCtxFname;
-    char*       pzData;
-    int         lineNo;
+    tScanCtx *    pCtx;
+    char*         pzScan;
+    char const *  pzCtxFname;
+    char *        pzData;
+    int           lineNo;
 };
 
 struct outSpec {
@@ -254,17 +261,18 @@ struct outSpec {
     char        zSuffix[ 1 ];
 };
 
-#define FPF_FREE       0x0001   /* free the fp structure   */
-#define FPF_UNLINK     0x0002   /* unlink file (temp file) */
-#define FPF_NOUNLINK   0x0004   /* do not unlink file      */
-#define FPF_STATIC_NM  0x0008   /* name statically alloced */
-#define FPF_NOCHMOD    0x0010   /* do not chmod(2) file    */
+#define FPF_FREE        0x0001   /* free the fp structure   */
+#define FPF_UNLINK      0x0002   /* unlink file (temp file) */
+#define FPF_NOUNLINK    0x0004   /* do not unlink file      */
+#define FPF_STATIC_NM   0x0008   /* name statically alloced */
+#define FPF_NOCHMOD     0x0010   /* do not chmod(2) file    */
+#define FPF_TEMPFILE    0x0020   /* the file is a temp      */
 
-struct fpStack {
-    int         flags;
-    tFpStack*   pPrev;
-    FILE*       pFile;
-    tCC*        pzOutName;
+struct fp_stack {
+    int                 flags;
+    tFpStack *          pPrev;
+    FILE *              pFile;
+    char const *        pzOutName;
 };
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
@@ -302,33 +310,84 @@ typedef struct {
 #define _MkStr(_s) #_s
 #define MK_STR(_s) _MkStr(_s)
 
+typedef struct {
+    int          line;
+    char const * file;
+    char const * text;
+} file_line_t;
+
+#define SCM_EVAL_CONST(_s) \
+    do { static file_line_t const fl = { __LINE__ - 1, __FILE__, _s }; \
+        pzLastScheme = fl.text; \
+        ag_scm_c_eval_string_from_file_line(fl.text, fl.file, fl.line); \
+    } while (0)
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  GLOBAL VARIABLES
  *
  *  General Processing Globals
  */
-#define pzProg   autogenOptions.pzProgName
-MODE teProcState procState        VALUE( PROC_STATE_INIT );
-MODE tTemplate*  pNamedTplList    VALUE( NULL );
-MODE tCC*        pzOopsPrefix     VALUE( "" );
+#define pzProg      autogenOptions.pzProgName
+MODE teProcState    procState        VALUE( PROC_STATE_INIT );
+MODE tTemplate*     pNamedTplList    VALUE( NULL );
+MODE char const *   pzOopsPrefix     VALUE( "" );
 
 /*
  *  Template Processing Globals
  */
-MODE tCC*        pzCurSfx         VALUE( NULL );
-MODE time_t      outTime          VALUE( 0 );
-MODE tFpStack*   pCurFp           VALUE( NULL );
-MODE tOutSpec*   pOutSpecList     VALUE( NULL );
-MODE jmp_buf     fileAbort;
-MODE char*       pzCurStart       VALUE( NULL );
-MODE uintptr_t   curStartOff      VALUE( 0 );
-MODE tForInfo    forInfo          VALUE( { 0 } );
-MODE FILE*       pfTrace          VALUE( NULL );
-MODE char*       pzTmpStderr      VALUE( NULL );
+MODE char const *   pzCurSfx         VALUE( NULL );
+/**
+ * The time to set for the modification times of the output files.
+ */
+MODE time_t         outTime          VALUE( 0 );
+/**
+ * The original time autogen started
+ */
+MODE time_t         startTime        VALUE( 0 );
+MODE tFpStack *     pCurFp           VALUE( NULL );
+MODE tOutSpec *     pOutSpecList     VALUE( NULL );
+MODE jmp_buf        fileAbort;
+MODE char *         pzCurStart       VALUE( NULL );
+MODE uintptr_t      curStartOff      VALUE( 0 );
+MODE tForInfo       forInfo          VALUE( { 0 } );
+MODE FILE *         pfTrace          VALUE( NULL );
+/**
+ * temporary file name template
+ */
+MODE char const *   pz_temp_tpl      VALUE( NULL );
+/**
+ * Length of the template that is the temp directory
+ */
+MODE size_t         temp_tpl_dir_len VALUE( 0 );
+/**
+ * dependency file file pointer.
+ */
+MODE FILE*          pfDepends        VALUE( NULL );
+/**
+ * name of target of rule
+ */
+MODE char const *   pzDepTarget      VALUE( NULL );
+/**
+ * name of dependency file
+ */
+MODE char const *   pzDepFile        VALUE( NULL );
+/**
+ * base name of both source and derived files.
+ * Either "_TList" or "_SList" gets put on the end.
+ */
+MODE char const *   pz_targ_base     VALUE( NULL );
+/**
+ * The actual list of input (source) files.
+ */
+MODE char const *   pzSourceList     VALUE( NULL );
+MODE size_t         source_size      VALUE( 0 );
+MODE size_t         source_used      VALUE( 0 );
+MODE ag_bool        dep_phonies      VALUE( AG_FALSE );
+MODE char *         cgi_stderr       VALUE( NULL );
 
-MODE tCC*        serverArgs[2]    VALUE( { NULL } );
-MODE tCC*        pzShellProgram   VALUE( MK_STR(CONFIG_SHELL) );
+MODE char const *   serverArgs[2]    VALUE( { NULL } );
+MODE char const *   pzShellProgram   VALUE( MK_STR(CONFIG_SHELL) );
 
 /*
  *  AutoGen definiton and template context
@@ -340,21 +399,20 @@ MODE tCC*        pzShellProgram   VALUE( MK_STR(CONFIG_SHELL) );
  *  the current set of macros is being extracted.
  *
  *  These are set in exactly ONE place:
- *  On entry to the dispatch routine (generateBlock)
- *  Two routines, however, must restore the values:  mFunc_Define
- *  and mFunc_For.  They are the only routines that dynamically
- *  push name/value pairs on the definition stack.
+ *  On entry to the dispatch routine (generateBlock).  Two routines, however,
+ *  must restore the values: mFunc_Define and mFunc_For.  They are the only
+ *  routines that dynamically push name/value pairs on the definition stack.
  */
-MODE tDefCtx     currDefCtx       VALUE( { NULL } );
-MODE tDefCtx     rootDefCtx       VALUE( { NULL } );
-MODE tTemplate*  pCurTemplate     VALUE( NULL );
-MODE tCC*        pzLastScheme     VALUE( NULL );
+MODE tDefCtx        currDefCtx       VALUE( { NULL } );
+MODE tDefCtx        rootDefCtx       VALUE( { NULL } );
+MODE tTemplate *    pCurTemplate     VALUE( NULL );
+MODE char const *   pzLastScheme     VALUE( NULL );
 #ifdef DAEMON_ENABLED
 /*
  *  When operating as a daemon, autogen can be told to reload
  *  its options the next time it wakes up (send it a SIGHUP).
  */
-MODE ag_bool     redoOptions      VALUE( AG_TRUE );
+MODE ag_bool        redoOptions      VALUE( AG_TRUE );
 #endif
 
 /*
@@ -367,69 +425,71 @@ MODE ag_bool     redoOptions      VALUE( AG_TRUE );
  *      alternation macros
  *  3.  mFunc_Case may transfer to one of its selection clauses.
  */
-MODE tMacro*     pCurMacro        VALUE( NULL );
+MODE tMacro *       pCurMacro        VALUE( NULL );
 
 /*
  *  Template Parsing Globals
  */
-MODE int         templLineNo      VALUE( 1 );
-MODE tScanCtx*   pBaseCtx         VALUE( NULL );
-MODE tScanCtx*   pCurCtx          VALUE( NULL );
-MODE tScanCtx*   pDoneCtx         VALUE( NULL );
-MODE size_t      endMacLen        VALUE( 0  );
-MODE char        zEndMac[   8 ]   VALUE( "" );
-MODE size_t      startMacLen      VALUE( 0  );
-MODE char        zStartMac[  8 ]  VALUE( "" );
-MODE int         guileFailure     VALUE( 0 );
+MODE int            templLineNo      VALUE( 1 );
+MODE tScanCtx *     pBaseCtx         VALUE( NULL );
+MODE tScanCtx *     pCurCtx          VALUE( NULL );
+MODE tScanCtx *     pDoneCtx         VALUE( NULL );
+MODE size_t         endMacLen        VALUE( 0  );
+MODE char           zEndMac[   8 ]   VALUE( "" );
+MODE size_t         startMacLen      VALUE( 0  );
+MODE char           zStartMac[  8 ]  VALUE( "" );
+MODE int            guileFailure     VALUE( 0 );
+MODE char           name_sep_ch      VALUE( '.' );
 
 /*
  *  Definition Parsing Globals
  */
-MODE char*       pzDefineData     VALUE( NULL );
-MODE char*       pz_token         VALUE( NULL );
-MODE te_dp_event lastToken        VALUE( DP_EV_INVALID );
+MODE char *         pzDefineData     VALUE( NULL );
+MODE char *         pz_token         VALUE( NULL );
+MODE te_dp_event    lastToken        VALUE( DP_EV_INVALID );
 
-MODE int         stackDepth       VALUE( 0 );
-MODE int         stackSize        VALUE( 16 );
-MODE tDefEntry*  parseStack[16]   VALUE( { 0 } );
-MODE tDefEntry** ppParseStack     VALUE( parseStack );
-MODE tDefEntry*  pCurrentEntry    VALUE( NULL );
+MODE int            stackDepth       VALUE( 0 );
+MODE int            stackSize        VALUE( 16 );
+MODE tDefEntry *    parseStack[16]   VALUE( { 0 } );
+MODE tDefEntry **   ppParseStack     VALUE( parseStack );
+MODE tDefEntry *    pCurrentEntry    VALUE( NULL );
+
+MODE tpChar         pCurDir          VALUE( NULL );
+
+MODE autogen_exit_code_t exit_code   VALUE( AUTOGEN_EXIT_OPTION_ERROR );
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  GLOBAL STRINGS
  */
-#define MKSTRING( name, val ) \
-        MODE char const z ## name[ sizeof( val )] VALUE( val )
+#define MKSTRING(_n, _v) \
+        MODE char const _n[sizeof(_v)] VALUE(_v)
 
-MKSTRING( AllocWhat,  "Could not allocate a %d byte %s\n" );
-MKSTRING( AllocErr,   "Allocation Failure" );
-MKSTRING( Cannot,     "fserr %d: cannot %s %s:  %s\n" );
-MKSTRING( TplWarn,    "Warning in template %s, line %d\n\t%s\n" );
-MKSTRING( FileLine,   "\tfrom %s line %d\n" );
-MKSTRING( ShDone,     "ShElL-OuTpUt-HaS-bEeN-cOmPlEtEd" );
-MKSTRING( NotStr,     "ERROR: %s is not a string\n" );
-MKSTRING( DevNull,    "/dev/null" );
-MKSTRING( ShellEnv,   "SHELL" );
-MKSTRING( Format,     "format" );
-MKSTRING( FileFormat, "%s%s\0%s.%s" );
-MKSTRING( Nil,        "" );
+MKSTRING(zAllocErr,   "Allocation Failure");
+MKSTRING(zAllocWhat,  "Could not allocate a %d byte %s\n");
+MKSTRING(zCannot,     "fserr %d: cannot %s %s:  %s\n");
+MKSTRING(zDevNull,    "/dev/null");
+MKSTRING(zEmpty,      "");
+MKSTRING(zFileFormat, "%s%s\0%s.%s");
+MKSTRING(zFileLine,   "\tfrom %s line %d\n");
+MKSTRING(zFormat,     "format");
+MKSTRING(zMemFile,    "in-mem file");
+MKSTRING(zNil,        "");
+MKSTRING(zNotStr,     "ERROR: %s is not a string\n");
+MKSTRING(zShDone,     "ShElL-OuTpUt-HaS-bEeN-cOmPlEtEd");
+MKSTRING(zTplWarn,    "Warning in template %s, line %d\n\t%s\n");
+
+#undef MKSTRING
 
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *
- *  GLOBAL PROCEDURES
- */
-#ifndef HAVE_STRLCPY
-extern size_t strlcpy( char* dest, char const* src, size_t n );
-#endif
-#ifdef ENABLE_FMEMOPEN
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  IF we have fopencookie or funopen, then we also have our own fmemopen
  */
-
-extern FILE * ag_fmemopen (void *buf, ssize_t len, char const *mode);
+#ifdef ENABLE_FMEMOPEN
+extern FILE * ag_fmemopen(void *buf, ssize_t len, char const *mode);
+extern int    ag_fmemioctl(FILE * fp, int req, ...);
 #endif
+
 #include "proto.h"
 
 typedef union {
@@ -443,6 +503,9 @@ MODE v2c_t p2p VALUE( { NULL } );
 #else
 # define AG_ABEND(s)  ag_abend_at(s)
 #endif
+#define  AG_CANT(_op, _wh) \
+    AG_ABEND(aprf(zCannot, errno, _op, _wh, strerror(errno)))
+
 #ifdef DEBUG_FSM
 # define DEBUG
 #else
@@ -452,13 +515,8 @@ MODE v2c_t p2p VALUE( { NULL } );
 #define AG_ABEND_IN(t,m,s) \
     STMTS( pCurTemplate=(t); pCurMacro=(m); AG_ABEND(s);)
 
-extern void  ag_scmStrings_init(   void );
-extern void  ag_scmStrings_deinit( void );
-extern void  ag_scmStrings_free(   void );
-extern char* ag_scribble( size_t size );
-
 #if defined(DEBUG_ENABLED)
-  extern void manageAllocatedData( void* pd );
+  extern void manageAllocatedData(void* pd);
 #else
 # define manageAllocatedData(_ptr)
 #endif
@@ -466,96 +524,7 @@ extern char* ag_scribble( size_t size );
 /*
  *  Code variations based on the version of Guile:
  */
-#if GUILE_VERSION < 107000  /* pre-Guile 1.7.x */
-
-#if GUILE_VERSION < 106000 /* Guile 1.4 */
-
-#   define AG_SCM_STRING_P(_s)          gh_string_p(_s)
-#   define AG_SCM_BOOL_P(_b)            gh_boolean_p(_b)
-#   define AG_SCM_SYM_P(_s)             gh_symbol_p(_s)
-#   define AG_SCM_IS_PROC(_p)           gh_procedure_p(_p)
-#   define AG_SCM_CHAR_P(_c)            gh_char_p(_c)
-#   define AG_SCM_VEC_P(_v)             gh_vector_p(_v)
-#   define AG_SCM_PAIR_P(_p)            gh_pair_p(_p)
-#   define AG_SCM_NUM_P(_n)             gh_number_p(_n)
-#   define AG_SCM_LIST_P(_l)            gh_list_p(_l)
-#   define AG_SCM_STR2SCM(_st,_sz)      gh_str2scm(_st,_sz)
-#   define AG_SCM_STR02SCM(_s)          gh_str02scm(_s)
-
-#else /* Guile 1.6.x */
-
-#   define AG_SCM_STRING_P(_s)          SCM_STRINGP(_s)
-#   define AG_SCM_BOOL_P(_b)            SCM_BOOLP(_b)
-#   define AG_SCM_SYM_P(_s)             SCM_SYMBOLP(_s)
-#   define AG_SCM_IS_PROC(_p)           SCM_NFALSEP( scm_procedure_p(_p))
-#   define AG_SCM_CHAR_P(_c)            SCM_CHARP(_c)
-#   define AG_SCM_VEC_P(_v)             SCM_VECTORP(_v)
-#   define AG_SCM_PAIR_P(_p)            SCM_NFALSEP( scm_pair_p(_p))
-#   define AG_SCM_NUM_P(_n)             SCM_NUMBERP(_n)
-#   define AG_SCM_LIST_P(_l)            SCM_NFALSEP( scm_list_p(_l))
-#   define AG_SCM_STR2SCM(_st,_sz)      scm_mem2string(_st,_sz)
-#   define AG_SCM_STR02SCM(_s)          scm_makfrom0str(_s)
-#endif
-
-  static inline char* ag_scm2zchars( SCM s, tCC* type )
-  {
-    if (! AG_SCM_STRING_P( s ))
-        AG_ABEND( aprf( zNotStr, type ));
-
-    if (SCM_SUBSTRP(s))
-        s = scm_makfromstr( SCM_CHARS(s), SCM_LENGTH(s), 0 );
-    return SCM_CHARS(s);
-  }
-
-# define AG_SCM_STRLEN(_s)              SCM_LENGTH(_s)
-# define AG_SCM_CHARS(_s)               SCM_CHARS(_s)
-
-#else /* Guile 1.7 and following */
-
-  extern char* ag_scm2zchars( SCM s, tCC* type );
-
-# if GUILE_VERSION < 108000
-#   define AG_SCM_STRLEN(_s)            SCM_STRING_LENGTH(_s)
-#   define AG_SCM_CHARS(_s)             SCM_STRING_CHARS(_s)
-#   define AG_SCM_VEC_P(_v)             SCM_VECTORP(_v)
-# else
-#   define AG_SCM_STRLEN(_s)            scm_c_string_length(_s)
-#   define AG_SCM_CHARS(_s)             scm_i_string_chars(_s)
-#   define AG_SCM_VEC_P(_v)             scm_is_vector(_v)
-# endif
-
-# define AG_SCM_STRING_P(_s)            scm_is_string(_s)
-# define AG_SCM_BOOL_P(_b)              SCM_BOOLP(_b)
-# define AG_SCM_SYM_P(_s)               SCM_SYMBOLP(_s)
-# define AG_SCM_IS_PROC(_p)             scm_is_true( scm_procedure_p(_p))
-# define AG_SCM_CHAR_P(_c)              SCM_CHARP(_c)
-# define AG_SCM_PAIR_P(_p)              scm_is_true( scm_pair_p(_p))
-# define AG_SCM_NUM_P(_n)               scm_is_number(_n)
-# define AG_SCM_LIST_P(_l)              SCM_NFALSEP( scm_list_p(_l))
-# define AG_SCM_STR2SCM(_st,_sz)        scm_from_locale_stringn(_st,_sz)
-# define AG_SCM_STR02SCM(_s)            scm_from_locale_string(_s)
-#endif /* pre-1.7 / 1.7 and following */
-
-/*
- *  Guile version invariant!!
- */
-#define AG_SCM_INT2SCM(_i)              gh_int2scm(_i)
-#define AG_SCM_SCM2INT(_i)              gh_scm2int(_i)
-#define AG_SCM_LONG2SCM(_i)             gh_long2scm(_i)
-#define AG_SCM_SCM2LONG(_i)             gh_scm2long(_i)
-
-static inline SCM ag_eval( tCC* pzStr )
-{
-    SCM res;
-    tCC* pzSaveScheme = pzLastScheme; /* Watch for nested calls */
-    pzLastScheme = pzStr;
-
-    res = ag_scm_c_eval_string_from_file_line(
-        pzStr, pCurTemplate->pzTplFile, pCurMacro->lineNo );
-
-    pzLastScheme = pzSaveScheme;
-    return res;
-}
+#include "guile-iface.h"
 
 #endif /* AUTOGEN_BUILD */
 /*
