@@ -2,12 +2,12 @@
 /*
  *  expMake.c
  *
- *  Time-stamp:        "2010-02-24 08:42:59 bkorb"
+ *  Time-stamp:        "2011-04-20 14:26:01 bkorb"
  *
  *  This module implements Makefile construction functions.
  *
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -93,63 +93,63 @@
  *  @end example
 =*/
 SCM
-ag_scm_makefile_script( SCM text )
+ag_scm_makefile_script(SCM text)
 {
-    tSCC   zNl[]  = " ; \\\n";
+    static char const zNl[]  = " ; \\\n";
 
     SCM    res;
-    char*  pzText = ag_scm2zchars( text, "GPL line prefix" );
+    char*  pzText = ag_scm2zchars(text, "GPL line prefix");
     char   tabch;
-    size_t sz     = strlen( pzText ) + 2;
+    size_t sz     = strlen(pzText) + 2;
 
     /*
      *  skip all blank lines and other initial white space
      *  in the source string.
      */
-    if (! isspace( *pzText ))
-        tabch = '\t';
+    if (! isspace(*pzText))
+        tabch = TAB;
     else {
-        while (isspace( *++pzText ))  ;
-        tabch  = (pzText[-1] == '\t') ? NUL : '\t';
+        while (isspace(*++pzText))  ;
+        tabch  = (pzText[-1] == TAB) ? NUL : TAB;
     }
 
     if (*pzText == NUL)
         return AG_SCM_STR02SCM(zNil);
 
     {
-        char*  pz  = strchr( pzText, '\n' );
-        size_t inc = ((*pzText == '\t') ? 0 : 1) + sizeof( zNl ) - 1;
+        char*  pz  = strchr(pzText, NL);
+        size_t inc = ((*pzText == TAB) ? 0 : 1) + sizeof(zNl) - 1;
 
         while (pz != NULL) {
             sz += inc;
-            pz = strchr( pz+1, '\n' );
+            pz = strchr(pz+1, NL);
         }
 
-        pz = strchr( pzText, '$' );
+        pz = strchr(pzText, '$');
         while (pz != NULL) {
             sz++;
-            pz = strchr( pz + 1, '$' );
+            pz = strchr(pz + 1, '$');
         }
     }
 
     {
-        char* pzRes = AGALOC( sz, "makefile text" );
+        char* pzRes = AGALOC(sz, "makefile text");
         char* pzOut = pzRes;
         char* pzScn = pzText;
 
         /*
          *  Force the initial line to start with a tab.
          */
-        *(pzOut++) = '\t';
+        *(pzOut++) = TAB;
 
         for (;;) {
             char ch = *(pzScn++);
             switch (ch) {
-            case '\n':
+            case NL:
                 /*
                  *  Backup past trailing white space (other than newline).
                  */
-                while (isspace( pzOut[ -1 ]) && (pzOut[ -1 ] != '\n'))
+                while (isspace(pzOut[ -1 ]) && (pzOut[ -1 ] != NL))
                     pzOut--;
 
                 /*
@@ -158,8 +158,8 @@ ag_scm_makefile_script( SCM text )
                  */
                 {
                     char* pz = pzScn;
-                    while (isspace( *pz )) {
-                        if (*(pz++) == '\n')
+                    while (isspace(*pz)) {
+                        if (*(pz++) == NL)
                             pzScn = pz;
                     }
                 }
@@ -176,7 +176,7 @@ ag_scm_makefile_script( SCM text )
                      *  The newline is already escaped, so don't
                      *  insert our extra command termination.
                      */
-                    *(pzOut++) = '\n';
+                    *(pzOut++) = NL;
                     break;
 
                 case '&':
@@ -195,8 +195,8 @@ ag_scm_makefile_script( SCM text )
                      *  Whatever the reason for a final '|' or ';' we will not
                      *  add a semi-colon after it.
                      */
-                    strcpy( pzOut, zNl + 2 );
-                    pzOut += sizeof( zNl ) - 3;
+                    strcpy(pzOut, zNl + 2);
+                    pzOut += sizeof(zNl) - 3;
                     break;
 
                 default:
@@ -204,8 +204,8 @@ ag_scm_makefile_script( SCM text )
                     /*
                      *  Terminate the current command and escape the newline.
                      */
-                    strcpy( pzOut, zNl );
-                    pzOut += sizeof( zNl ) - 1;
+                    strcpy(pzOut, zNl);
+                    pzOut += sizeof(zNl) - 1;
                 }
 
                 /*
@@ -252,8 +252,8 @@ ag_scm_makefile_script( SCM text )
         } copy_done:;
 
         sz    = (pzOut - pzRes);
-        res   = AG_SCM_STR2SCM( pzRes, sz );
-        AGFREE( pzRes );
+        res   = AG_SCM_STR2SCM(pzRes, sz);
+        AGFREE(pzRes);
     }
 
     return res;
