@@ -1,15 +1,14 @@
 
-/*
- *  expString.c
- *  $Id: abe96753939787ee59fb6bebac66f346cc0c5e70 $
+/**
+ * \file expString.c
  *
- *  Time-stamp:        "2009-11-01 14:05:40 bkorb"
+ *  Time-stamp:        "2011-04-20 14:25:50 bkorb"
  *
  *  This module implements expression functions that
  *  manipulate string values.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -26,36 +25,29 @@
  */
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static size_t
 string_size(char const * pzScn, size_t newLineSize);
 
 static SCM
-makeString( tCC*    pzText,
-            tCC*    pzNewLine,
-            size_t  newLineSize );
+makeString(char const * pzText, char const * pzNewLine, size_t newLineSize);
 
 static size_t
-stringify_for_shell(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize);
+stringify_for_sh(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize);
 
 static SCM
-shell_stringify( SCM obj, u_int qt );
+shell_stringify(SCM obj, u_int qt);
+
+static int
+sub_count(char const * haystack, char const * needle);
 
 static void
 do_substitution(
-    tCC*        pzStr,
-    scm_sizet   strLen,
-    SCM         match,
-    SCM         repl,
-    char**      ppzRes,
-    scm_sizet*  pResLen );
-
-static void
-do_multi_subs(
-    char**      ppzStr,
-    scm_sizet*  pStrLen,
-    SCM         match,
-    SCM         repl );
+    char const * pzStr,
+    scm_sizet    strLen,
+    SCM          match,
+    SCM          repl,
+    char **      ppzRes,
+    scm_sizet *  pResLen );
 /* = = = END-STATIC-FORWARD = = = */
 
 static size_t
@@ -64,7 +56,7 @@ string_size(char const * pzScn, size_t newLineSize)
     /*
      *  Start by counting the start and end quotes, plus the NUL.
      */
-    size_t dtaSize = sizeof( "\"\"" );
+    size_t dtaSize = sizeof("\"\"");
 
     for (;;) {
         char ch = *(pzScn++);
@@ -86,11 +78,11 @@ string_size(char const * pzScn, size_t newLineSize)
         case NUL:
             return dtaSize;
 
-        case '\n':
+        case NL:
             dtaSize += newLineSize;
             break;
 
-        case '\t':
+        case TAB:
         case '\a':
         case '\b':
         case '\f':
@@ -106,14 +98,12 @@ string_size(char const * pzScn, size_t newLineSize)
 }
 
 static SCM
-makeString( tCC*    pzText,
-            tCC*    pzNewLine,
-            size_t  newLineSize )
+makeString(char const * pzText, char const * pzNewLine, size_t newLineSize)
 {
-    char     z[ SCRIBBLE_SIZE ];
+    char     z[SCRIBBLE_SIZE];
     char*    pzDta;
     char*    pzFre;
-    tCC*     pzScn   = pzText;
+    char const * pzScn   = pzText;
     size_t   dtaSize = string_size(pzText, newLineSize);
 
     /*
@@ -121,7 +111,7 @@ makeString( tCC*    pzText,
      *  Allocate what we need.
      */
     if (dtaSize >= sizeof(z))
-         pzFre = pzDta = AGALOC( dtaSize, "quoting string" );
+         pzFre = pzDta = AGALOC(dtaSize, "quoting string");
     else pzFre = pzDta = z;
 
     *(pzDta++) = '"';
@@ -140,11 +130,11 @@ makeString( tCC*    pzText,
         case NUL:
             goto copyDone;
 
-        case '\n':
+        case NL:
             /*
              *  place contiguous new-lines on a single line
              */
-            while (pzScn[1] == '\n') {
+            while (pzScn[1] == NL) {
                 *(pzDta++) = '\\';
                 *(pzDta++) = 'n';
                 pzScn++;
@@ -163,7 +153,7 @@ makeString( tCC*    pzText,
                 goto copyDone;
             }
 
-            strcpy( pzDta, pzNewLine );
+            strcpy(pzDta, pzNewLine);
             pzDta += newLineSize;
             break;
 
@@ -187,7 +177,7 @@ makeString( tCC*    pzText,
             *(pzDta++) = 'r';
             break;
 
-        case '\t':
+        case TAB:
             *(pzDta++) = '\\';
             *(pzDta++) = 't';
             break;
@@ -199,12 +189,12 @@ makeString( tCC*    pzText,
 
         default:
         {
-            tSCC zFmt[] = "\\%03o";
+            static char const zFmt[] = "\\%03o";
             /*
              *  sprintf is safe here, because we already computed
              *  the amount of space we will be using.
              */
-            sprintf( pzDta, zFmt, ch );
+            sprintf(pzDta, zFmt, ch);
             pzDta += 4;
         }
         }
@@ -221,15 +211,15 @@ makeString( tCC*    pzText,
     *pzDta = NUL;
 
     {
-        SCM res = scm_makfrom0str( pzFre );
+        SCM res = AG_SCM_STR02SCM(pzFre);
         if (pzFre != z)
-            AGFREE( pzFre );
+            AGFREE(pzFre);
         return res;
     }
 }
 
 static size_t
-stringify_for_shell(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize)
+stringify_for_sh(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize)
 {
     char * pz = pzNew;
     *(pz++) = qt;
@@ -291,11 +281,11 @@ stringify_for_shell(char * pzNew, u_int qt, char const * pzDta, size_t dtaSize)
 }
 
 static SCM
-shell_stringify( SCM obj, u_int qt )
+shell_stringify(SCM obj, u_int qt)
 {
     char*  pzNew;
     size_t dtaSize = 3;
-    char*  pzDta   = ag_scm2zchars( obj, "AG Object" );
+    char*  pzDta   = ag_scm2zchars(obj, "AG Object");
     char*  pz      = pzDta;
 
     for (;;) {
@@ -314,16 +304,30 @@ shell_stringify( SCM obj, u_int qt )
         }
     } loopDone1:;
 
-    pzNew = AGALOC( dtaSize, "shell string" );
-    dtaSize = stringify_for_shell(pzNew, qt, pzDta, dtaSize);
+    pzNew = AGALOC(dtaSize, "shell string");
+    dtaSize = stringify_for_sh(pzNew, qt, pzDta, dtaSize);
 
     {
-        SCM res = AG_SCM_STR2SCM( pzNew, dtaSize );
-        AGFREE( pzNew );
+        SCM res = AG_SCM_STR2SCM(pzNew, dtaSize);
+        AGFREE(pzNew);
         return res;
     }
 }
 
+static int
+sub_count(char const * haystack, char const * needle)
+{
+    int repCt = 0;
+    size_t needle_len = strlen(needle);
+
+    for (;;) {
+        haystack = strstr(haystack, needle);
+        if (haystack == NULL) break;
+        repCt++;
+        haystack += needle_len;
+    }
+    return repCt;
+}
 
 /* * * * *
  *
@@ -333,31 +337,23 @@ shell_stringify( SCM obj, u_int qt )
  */
 static void
 do_substitution(
-    tCC*        pzStr,
-    scm_sizet   strLen,
-    SCM         match,
-    SCM         repl,
-    char**      ppzRes,
-    scm_sizet*  pResLen )
+    char const * pzStr,
+    scm_sizet    strLen,
+    SCM          match,
+    SCM          repl,
+    char **      ppzRes,
+    scm_sizet *  pResLen )
 {
-    char* pzMatch = ag_scm2zchars( match, "match text" );
-    char* pzRepl  = ag_scm2zchars( repl,  "repl text" );
-    int   matchL  = AG_SCM_STRLEN( match );
-    int   replL   = AG_SCM_STRLEN( repl );
+    char* pzMatch = ag_scm2zchars(match, "match text");
+    char* pzRepl  = ag_scm2zchars(repl,  "repl text");
+    int   matchL  = AG_SCM_STRLEN(match);
+    int   replL   = AG_SCM_STRLEN(repl);
     int   endL    = strLen;
 
-    int   repCt   = 0;
-    tCC*  pz      = pzStr;
-    char* pzRes;
-    char* pzScan;
-
-    for (;;) {
-        tCC* pzNxt = strstr( pz, pzMatch );
-        if (pzNxt == NULL)
-            break;
-        repCt++;
-        pz = pzNxt + matchL;
-    }
+    int   repCt   = sub_count(pzStr, pzMatch);
+    char const * pz = pzStr;
+    char * pzRes;
+    char * pzScan;
 
     /*
      *  No substitutions -- no work.  The caller will have to track
@@ -367,24 +363,24 @@ do_substitution(
         return;
 
     endL   = endL + (replL * repCt) - (matchL * repCt);
-    pzScan = pzRes = AGALOC( endL + 1, "substitution" );
+    pzScan = pzRes = ag_scribble(endL + 1);
     pz     = pzStr;
 
     for (;;) {
-        tCC* pzNxt = strstr( pz, pzMatch );
+        char const * pzNxt = strstr(pz, pzMatch);
         if (pzNxt == NULL)
             break;
         if (pz != pzNxt) {
-            memcpy( pzScan, pz, (size_t)(pzNxt - pz) );
+            memcpy(pzScan, pz, (size_t)(pzNxt - pz));
             pzScan += (pzNxt - pz);
         }
-        memcpy( pzScan, pzRepl, (size_t)replL );
+        memcpy(pzScan, pzRepl, (size_t)replL);
         pzScan += replL;
         pz = pzNxt + matchL;
     }
-    strcpy( pzScan, pz );
+    strcpy(pzScan, pz);
     *ppzRes = pzRes;
-    *pResLen = (pzScan - pzRes) + strlen( pz );
+    *pResLen = (pzScan - pzRes) + strlen(pz);
 }
 
 
@@ -394,48 +390,35 @@ do_substitution(
  *  be done in the order found in the tree walk of list values.
  *  The "match" and "repl" trees *must* be identical in structure.
  */
-static void
-do_multi_subs(
-    char**      ppzStr,
-    scm_sizet*  pStrLen,
-    SCM         match,
-    SCM         repl )
+LOCAL void
+do_multi_subs(char ** ppzStr, scm_sizet * pStrLen, SCM match, SCM repl)
 {
     char* pzStr = *ppzStr;
-    char* pzOri = pzStr;
     char* pzNxt = pzStr;
 
     /*
      *  Loop for as long as our list has more entries
      */
-    while (! SCM_NULLP( match )) {
+    while (! AG_SCM_NULLP(match)) {
         /*
          *  "CAR" is the current value, "CDR" is rest of list
          */
-        SCM  matchCar  = SCM_CAR( match );
-        SCM  replCar   = SCM_CAR( repl );
+        SCM  matchCar  = SCM_CAR(match);
+        SCM  replCar   = SCM_CAR(repl);
 
-        match = SCM_CDR( match );
-        repl  = SCM_CDR( repl );
+        match = SCM_CDR(match);
+        repl  = SCM_CDR(repl);
 
-        if (AG_SCM_STRING_P( matchCar )) {
-            do_substitution( pzStr, *pStrLen, matchCar, replCar,
-                             &pzNxt, pStrLen );
-
-            /*
-             *  Be sure to free intermediate results.
-             *  passed-in values are freed by the caller.
-             */
-            if ((pzStr != pzOri) && (pzStr != pzNxt))
-                // coverity[freed_arg] -- invalid alias analysis
-                AGFREE( pzStr );
+        if (AG_SCM_STRING_P(matchCar)) {
+            do_substitution(pzStr, *pStrLen, matchCar, replCar,
+                            &pzNxt, pStrLen);
 
             // coverity[use_after_free] -- invalid alias analysis
             pzStr = pzNxt;
         }
 
-        else if (gh_list_p( matchCar ))
-            do_multi_subs( &pzStr, pStrLen, matchCar, replCar );
+        else if (AG_SCM_LIST_P(matchCar))
+            do_multi_subs(&pzStr, pStrLen, matchCar, replCar);
 
         else
             /*
@@ -462,31 +445,31 @@ do_multi_subs(
  *      in one of the entries in the second (list-of-strings) argument.
 =*/
 SCM
-ag_scm_in_p( SCM obj, SCM list )
+ag_scm_in_p(SCM obj, SCM list)
 {
     int     len;
     size_t  lenz;
     SCM     car;
-    tCC *   pz1;
+    char const * pz1;
 
-    if (! AG_SCM_STRING_P( obj ))
+    if (! AG_SCM_STRING_P(obj))
         return SCM_UNDEFINED;
 
-    pz1  = AG_SCM_CHARS(  obj );
-    lenz = AG_SCM_STRLEN( obj );
+    pz1  = AG_SCM_CHARS(obj);
+    lenz = AG_SCM_STRLEN(obj);
 
     /*
      *  If the second argument is a string somehow, then treat
      *  this as a straight out string comparison
      */
-    if (AG_SCM_STRING_P( list )) {
-        if (  (AG_SCM_STRLEN( list ) == lenz)
-           && (strncmp( pz1, AG_SCM_CHARS( list ), lenz) == 0))
+    if (AG_SCM_STRING_P(list)) {
+        if (  (AG_SCM_STRLEN(list) == lenz)
+           && (strncmp(pz1, AG_SCM_CHARS(list), lenz) == 0))
             return SCM_BOOL_T;
         return SCM_BOOL_F;
     }
 
-    len = scm_ilength( list );
+    len = scm_ilength(list);
     if (len == 0)
         return SCM_BOOL_F;
 
@@ -494,8 +477,8 @@ ag_scm_in_p( SCM obj, SCM list )
      *  Search all the lists and sub-lists passed in
      */
     while (len-- > 0) {
-        car  = SCM_CAR( list );
-        list = SCM_CDR( list );
+        car  = SCM_CAR(list);
+        list = SCM_CDR(list);
 
         /*
          *  This routine is listed as getting a list as the second
@@ -503,14 +486,14 @@ ag_scm_in_p( SCM obj, SCM list )
          *  hands it to us, it magically becomes a nested list.
          *  This unravels that.
          */
-        if (! AG_SCM_STRING_P( car )) {
-            if (ag_scm_in_p( obj, car ) == SCM_BOOL_T)
+        if (! AG_SCM_STRING_P(car)) {
+            if (ag_scm_in_p(obj, car) == SCM_BOOL_T)
                 return SCM_BOOL_T;
             continue;
         }
 
-        if (  (AG_SCM_STRLEN( car ) == lenz)
-           && (strncmp( pz1, AG_SCM_CHARS( car ), lenz ) == 0) )
+        if (  (AG_SCM_STRLEN(car) == lenz)
+           && (strncmp(pz1, AG_SCM_CHARS(car), lenz) == 0) )
             return SCM_BOOL_T;
     }
 
@@ -531,7 +514,7 @@ ag_scm_in_p( SCM obj, SCM list )
  *       cannot always control that.
 =*/
 SCM
-ag_scm_join( SCM sep, SCM list )
+ag_scm_join(SCM sep, SCM list)
 {
     int        l_len, sv_l_len;
     SCM        car;
@@ -539,26 +522,26 @@ ag_scm_join( SCM sep, SCM list )
     size_t     sep_len;
     scm_sizet  str_len;
     char*      pzRes;
-    tCC *      pzSep;
+    char const * pzSep;
     char*      pzScan;
 
-    if (! AG_SCM_STRING_P( sep ))
+    if (! AG_SCM_STRING_P(sep))
         return SCM_UNDEFINED;
 
-    sv_l_len = l_len = scm_ilength( list );
+    sv_l_len = l_len = scm_ilength(list);
     if (l_len == 0)
         return AG_SCM_STR02SCM(zNil);
 
-    pzSep   = AG_SCM_CHARS( sep );
-    sep_len = AG_SCM_STRLEN( sep );
+    pzSep   = AG_SCM_CHARS(sep);
+    sep_len = AG_SCM_STRLEN(sep);
     str_len = 0;
 
     /*
      *  Count up the lengths of all the strings to be joined.
      */
     for (;;) {
-        car  = SCM_CAR( list );
-        list = SCM_CDR( list );
+        car  = SCM_CAR(list);
+        list = SCM_CDR(list);
 
         /*
          *  This routine is listed as getting a list as the second
@@ -566,14 +549,14 @@ ag_scm_join( SCM sep, SCM list )
          *  hands it to us, it magically becomes a nested list.
          *  This unravels that.
          */
-        if (! AG_SCM_STRING_P( car )) {
+        if (! AG_SCM_STRING_P(car)) {
             if (car != SCM_UNDEFINED)
-                car = ag_scm_join( sep, car );
-            if (! AG_SCM_STRING_P( car ))
+                car = ag_scm_join(sep, car);
+            if (! AG_SCM_STRING_P(car))
                 return SCM_UNDEFINED;
         }
 
-        str_len += AG_SCM_STRLEN( car );
+        str_len += AG_SCM_STRLEN(car);
 
         if (--l_len <= 0)
             break;
@@ -582,7 +565,7 @@ ag_scm_join( SCM sep, SCM list )
     }
 
     l_len = sv_l_len;
-    pzRes = pzScan = ag_scribble( str_len );
+    pzRes = pzScan = ag_scribble(str_len);
 
     /*
      *  Now, copy each one into the output
@@ -590,17 +573,17 @@ ag_scm_join( SCM sep, SCM list )
     for (;;) {
         size_t cpy_len;
 
-        car   = SCM_CAR( alist );
-        alist = SCM_CDR( alist );
+        car   = SCM_CAR(alist);
+        alist = SCM_CDR(alist);
 
         /*
          *  This unravels nested lists.
          */
-        if (! AG_SCM_STRING_P( car ))
-            car = ag_scm_join( sep, car );
+        if (! AG_SCM_STRING_P(car))
+            car = ag_scm_join(sep, car);
 
-        cpy_len = AG_SCM_STRLEN( car );
-        memcpy( (void*)pzScan, AG_SCM_CHARS( car ), cpy_len );
+        cpy_len = AG_SCM_STRLEN(car);
+        memcpy((void*)pzScan, AG_SCM_CHARS(car), cpy_len);
         pzScan += cpy_len;
 
         /*
@@ -608,11 +591,11 @@ ag_scm_join( SCM sep, SCM list )
          */
         if (--l_len <= 0)
             break;
-        memcpy( (void*)pzScan, (void*)pzSep, sep_len );
+        memcpy((void*)pzScan, (void*)pzSep, sep_len);
         pzScan += sep_len;
     }
 
-    return AG_SCM_STR2SCM( pzRes, str_len );
+    return AG_SCM_STR2SCM(pzRes, str_len);
 }
 
 
@@ -640,7 +623,7 @@ ag_scm_join( SCM sep, SCM list )
  *  @end example
 =*/
 SCM
-ag_scm_prefix( SCM prefix, SCM text )
+ag_scm_prefix(SCM prefix, SCM text)
 {
     char*    pzPfx;
     char*    pzText;
@@ -649,17 +632,17 @@ ag_scm_prefix( SCM prefix, SCM text )
     size_t   pfx_size;
     char*    pzRes;
 
-    pzPfx   = ag_scm2zchars( prefix, "prefix" );
+    pzPfx   = ag_scm2zchars(prefix, "prefix");
     pzDta   = \
-    pzText  = ag_scm2zchars( text, "text" );
-    pfx_size = strlen( pzPfx );
+    pzText  = ag_scm2zchars(text, "text");
+    pfx_size = strlen(pzPfx);
     out_size = pfx_size;
 
     for (;;) {
         switch (*(pzText++)) {
         case NUL:
             goto exit_count;
-        case '\n':
+        case NL:
             out_size += pfx_size;
             /* FALLTHROUGH */
         default:
@@ -668,8 +651,8 @@ ag_scm_prefix( SCM prefix, SCM text )
     } exit_count:;
 
     pzText = pzDta;
-    pzRes  = pzDta = ag_scribble( rem_size = out_size );
-    strcpy( pzDta, pzPfx );
+    pzRes  = pzDta = ag_scribble(rem_size = out_size);
+    strcpy(pzDta, pzPfx);
     pzDta    += pfx_size;
     rem_size -= pfx_size;
     pfx_size++;
@@ -679,13 +662,13 @@ ag_scm_prefix( SCM prefix, SCM text )
         switch (ch) {
         case NUL:
             if (rem_size != 0)
-                AG_ABEND( "(prefix ...) failed" );
+                AG_ABEND("(prefix ...) failed");
 
-            return AG_SCM_STR2SCM( pzRes, out_size );
+            return AG_SCM_STR2SCM(pzRes, out_size);
 
-        case '\n':
+        case NL:
             *pzDta    = ch;
-            strcpy( pzDta+1, pzPfx );
+            strcpy(pzDta+1, pzPfx);
             pzDta    += pfx_size;
             rem_size -= pfx_size;
             break;
@@ -721,14 +704,14 @@ ag_scm_prefix( SCM prefix, SCM text )
  *  the new command is issued.
 =*/
 SCM
-ag_scm_shell( SCM cmd )
+ag_scm_shell(SCM cmd)
 {
-    if (! AG_SCM_STRING_P( cmd ))
+    if (! AG_SCM_STRING_P(cmd))
         return SCM_UNDEFINED;
     {
-        char* pz = runShell( ag_scm2zchars( cmd, "command" ));
-        cmd   = AG_SCM_STR02SCM( pz );
-        AGFREE( (void*)pz );
+        char* pz = runShell(ag_scm2zchars(cmd, "command"));
+        cmd   = AG_SCM_STR02SCM(pz);
+        AGFREE((void*)pz);
         return cmd;
     }
 }
@@ -746,22 +729,22 @@ ag_scm_shell( SCM cmd )
  *       then send the result to the shell for interpretation.
 =*/
 SCM
-ag_scm_shellf( SCM fmt, SCM alist )
+ag_scm_shellf(SCM fmt, SCM alist)
 {
-    int   len = scm_ilength( alist );
+    int   len = scm_ilength(alist);
     char* pz;
 
 #ifdef DEBUG_ENABLED
     if (len < 0)
-        AG_ABEND( "invalid alist to shellf" );
+        AG_ABEND("invalid alist to shellf");
 #endif
 
-    pz  = ag_scm2zchars( fmt, "format" );
-    fmt = run_printf( pz, len, alist );
+    pz  = ag_scm2zchars(fmt, "format");
+    fmt = run_printf(pz, len, alist);
 
-    pz  = runShell( ag_scm2zchars( fmt, "shell script" ));
-    fmt = AG_SCM_STR02SCM( pz );
-    AGFREE( (void*)pz );
+    pz  = runShell(ag_scm2zchars(fmt, "shell script"));
+    fmt = AG_SCM_STR02SCM(pz);
+    AGFREE((void*)pz);
     return fmt;
 }
 
@@ -786,25 +769,25 @@ ag_scm_shellf( SCM fmt, SCM alist )
  *  conventional ASCII strings.
 =*/
 SCM
-ag_scm_raw_shell_str( SCM obj )
+ag_scm_raw_shell_str(SCM obj)
 {
     char*      pzDta;
     char*      pz;
     char*      pzFree;
 
-    pzDta = ag_scm2zchars( obj, "AG Object" );
+    pzDta = ag_scm2zchars(obj, "AG Object");
 
     {
-        size_t dtaSize = AG_SCM_STRLEN( obj ) + 3; /* NUL + 2 quotes */
+        size_t dtaSize = AG_SCM_STRLEN(obj) + 3; /* NUL + 2 quotes */
         pz = pzDta-1;
         for (;;) {
-            pz = strchr( pz+1, '\'' );
+            pz = strchr(pz+1, '\'');
             if (pz == NULL)
                 break;
             dtaSize += 3; /* '\'' -> 3 additional chars */
         }
 
-        pzFree = pz = AGALOC( dtaSize + 2, "raw string" );
+        pzFree = pz = AGALOC(dtaSize + 2, "raw string");
     }
 
     /*
@@ -856,8 +839,8 @@ ag_scm_raw_shell_str( SCM obj )
 
  returnString:
     {
-        SCM res = scm_makfrom0str( pzFree );
-        AGFREE( pzFree );
+        SCM res = AG_SCM_STR02SCM(pzFree);
+        AGFREE(pzFree);
         return res;
     }
 }
@@ -877,7 +860,7 @@ ag_scm_raw_shell_str( SCM obj )
  *  escape character @code{\\} before two special characters to
  *  accomplish this: the backslash @code{\\} and double quote @code{"}.
  *
- *  @strong{NOTE}: some shells will not correctly handle unusual
+ *  @strong{Notice}: some shells will not correctly handle unusual
  *  non-printing characters.  This routine works for most reasonably
  *  conventional ASCII strings.
  *
@@ -928,9 +911,9 @@ ag_scm_raw_shell_str( SCM obj )
  *  three for the "cooked" string examples.
 =*/
 SCM
-ag_scm_shell_str( SCM obj )
+ag_scm_shell_str(SCM obj)
 {
-    return shell_stringify( obj, (unsigned)'"' );
+    return shell_stringify(obj, (unsigned)'"');
 }
 
 /*=gfunc sub_shell_str
@@ -946,9 +929,9 @@ ag_scm_shell_str( SCM obj )
  *   character is @code{"}.
 =*/
 SCM
-ag_scm_sub_shell_str( SCM obj )
+ag_scm_sub_shell_str(SCM obj)
 {
-    return shell_stringify( obj, (unsigned)'`' );
+    return shell_stringify(obj, (unsigned)'`');
 }
 
 
@@ -962,7 +945,7 @@ ag_scm_sub_shell_str( SCM obj )
  *       with a name.  They must all be text values or we choke.
 =*/
 SCM
-ag_scm_stack( SCM obj )
+ag_scm_stack(SCM obj)
 {
     SCM         res;
     SCM *       pos = &res;
@@ -972,7 +955,7 @@ ag_scm_stack( SCM obj )
 
     res = SCM_EOL;
 
-    ppDE = findEntryList( ag_scm2zchars( obj, "AG Object" ));
+    ppDE = findEntryList(ag_scm2zchars(obj, "AG Object"));
     if (ppDE == NULL)
         return SCM_EOL;
 
@@ -985,9 +968,9 @@ ag_scm_stack( SCM obj )
         if (pDE->valType != VALTYP_TEXT)
             return SCM_UNDEFINED;
 
-        str  = AG_SCM_STR02SCM( pDE->val.pzText );
-        *pos = scm_cons( str, SCM_EOL );
-        pos  = SCM_CDRLOC( *pos );
+        str  = AG_SCM_STR02SCM(pDE->val.pzText);
+        *pos = scm_cons(str, SCM_EOL);
+        pos  = SCM_CDRLOC(*pos);
     }
 
     return res;
@@ -1009,12 +992,12 @@ ag_scm_stack( SCM obj )
  *  backslash-n-backslash and newline sequence,
 =*/
 SCM
-ag_scm_kr_string( SCM str )
+ag_scm_kr_string(SCM str)
 {
-    tSCC zNewLine[] = "\\n\\\n";
+    static char const zNewLine[] = "\\n\\\n";
 
-    return makeString( ag_scm2zchars( str, "string" ),
-                       zNewLine, sizeof( zNewLine )-1 );
+    return makeString(ag_scm2zchars(str, "string"),
+                      zNewLine, sizeof(zNewLine)-1);
 }
 
 
@@ -1038,12 +1021,12 @@ ag_scm_kr_string( SCM str )
  *
 =*/
 SCM
-ag_scm_c_string( SCM str )
+ag_scm_c_string(SCM str)
 {
-    tSCC zNewLine[] = "\\n\"\n       \"";
+    static char const zNewLine[] = "\\n\"\n       \"";
 
-    return makeString( ag_scm2zchars( str, "string" ),
-                       zNewLine, sizeof( zNewLine )-1 );
+    return makeString(ag_scm2zchars(str, "string"),
+                      zNewLine, sizeof(zNewLine)-1);
 }
 
 
@@ -1065,18 +1048,18 @@ ag_scm_c_string( SCM str )
  *      and incompatible implementations!
 =*/
 SCM
-ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
+ag_scm_string_tr_x(SCM str, SCM from_xform, SCM to_xform)
 {
-    unsigned char map[ 1 << 8 /* bits-per-byte */ ];
-    int   i      = sizeof(map) - 1;
-    char* pzFrom = ag_scm2zchars( from_xform, "string" );
-    char* pzTo   = ag_scm2zchars(   to_xform, "string" );
+    unsigned char ch_map[ 1 << 8 /* bits-per-byte */ ];
+    int   i      = sizeof(ch_map) - 1;
+    char* pzFrom = ag_scm2zchars(from_xform, "string");
+    char* pzTo   = ag_scm2zchars(to_xform, "string");
 
     do  {
-        map[i] = i;
+        ch_map[i] = i;
     } while (--i > 0);
 
-    for (;i <= sizeof(map) - 1;i++) {
+    for (;i <= sizeof(ch_map) - 1;i++) {
         unsigned char fch = (unsigned char)*(pzFrom++);
         unsigned char tch = (unsigned char)*(pzTo++);
 
@@ -1097,7 +1080,7 @@ ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
                 unsigned char te = (unsigned char)pzTo[0];
                 if (te != NUL) {
                     while (fs < fe) {
-                        map[ fs++ ] = ts;
+                        ch_map[ fs++ ] = ts;
                         if (ts < te) ts++;
                     }
                     break;
@@ -1105,14 +1088,14 @@ ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
             }
 
         default:
-            map[ fch ] = tch;
+            ch_map[ fch ] = tch;
         }
     } mapDone:;
 
-    pzTo = (char*)(void*)AG_SCM_CHARS( str );
-    i    = AG_SCM_STRLEN( str );
+    pzTo = (char*)(void*)AG_SCM_CHARS(str);
+    i    = AG_SCM_STRLEN(str);
     while (i-- > 0) {
-        *pzTo = map[ (int)*pzTo ];
+        *pzTo = ch_map[ (int)*pzTo ];
         pzTo++;
     }
     return str;
@@ -1132,11 +1115,11 @@ ag_scm_string_tr_x( SCM str, SCM from_xform, SCM to_xform )
  *      over-write the previous value.
 =*/
 SCM
-ag_scm_string_tr( SCM Str, SCM From, SCM To )
+ag_scm_string_tr(SCM Str, SCM From, SCM To)
 {
-    scm_sizet lenz  = AG_SCM_STRLEN( Str );
-    SCM       res   = AG_SCM_STR2SCM( AG_SCM_CHARS(Str), lenz );
-    return ag_scm_string_tr_x( res, From, To );
+    scm_sizet lenz  = AG_SCM_STRLEN(Str);
+    SCM       res   = AG_SCM_STR2SCM(AG_SCM_CHARS(Str), lenz);
+    return ag_scm_string_tr_x(res, From, To);
 }
 
 
@@ -1161,32 +1144,24 @@ ag_scm_string_tr( SCM Str, SCM From, SCM To )
  * @end example
 =*/
 SCM
-ag_scm_string_substitute( SCM Str, SCM Match, SCM Repl )
+ag_scm_string_substitute(SCM Str, SCM Match, SCM Repl)
 {
-    tCC*       pzStr;
+    char const * pzStr;
     scm_sizet  len;
     SCM        res;
 
-    if (! AG_SCM_STRING_P( Str ))
+    if (! AG_SCM_STRING_P(Str))
         return SCM_UNDEFINED;
 
-    pzStr = AG_SCM_CHARS( Str );
-    len   = AG_SCM_STRLEN( Str );
+    pzStr = AG_SCM_CHARS(Str);
+    len   = AG_SCM_STRLEN(Str);
 
-    if (AG_SCM_STRING_P( Match ))
-        do_substitution( pzStr, len, Match, Repl,
-                         (char**)&pzStr, &len );
+    if (AG_SCM_STRING_P(Match))
+        do_substitution(pzStr, len, Match, Repl, (char**)&pzStr, &len);
     else
-        do_multi_subs( (char**)&pzStr, &len, Match, Repl );
+        do_multi_subs((char**)&pzStr, &len, Match, Repl);
 
-    res = AG_SCM_STR2SCM( pzStr, len );
-
-    /*
-     *  IF we have an allocated intermediate result,
-     *  THEN we must free it.  We might have our original string.
-     */
-    if (pzStr != AG_SCM_CHARS( Str ))
-        AGFREE( (char*)pzStr );
+    res = AG_SCM_STR2SCM(pzStr, len);
     return res;
 }
 
@@ -1205,17 +1180,17 @@ ag_scm_string_substitute( SCM Str, SCM Match, SCM Repl )
  *         @code{HH:MM:SS} or, without hours, as @code{MM:SS}.
 =*/
 SCM
-ag_scm_time_string_to_number( SCM time_spec )
+ag_scm_time_string_to_number(SCM time_spec)
 {
     extern time_t parse_duration(char const * in_pz);
 
-    tCC *   pz;
+    char const * pz;
     time_t  time_period;
 
-    if (! AG_SCM_STRING_P( time_spec ))
+    if (! AG_SCM_STRING_P(time_spec))
         return SCM_UNDEFINED;
 
-    pz = AG_SCM_CHARS( time_spec );
+    pz = AG_SCM_CHARS(time_spec);
     time_period = parse_duration(pz);
     
     return AG_SCM_INT2SCM((int)time_period);
