@@ -4,10 +4,10 @@
  *
  *  This module implements the DEFINE text function.
  *
- *  Time-stamp:        "2010-07-28 18:42:39 bkorb"
+ *  Time-stamp:        "2011-04-21 16:51:41 bkorb"
  *
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -208,7 +208,7 @@ fill_in_values(tDefList * pDL, char * pzScan, tTemplate* pT, tMacro* pMac)
         default:
             AG_ABEND_IN(pT, pMac, "name not followed by '='");
 
-        case ' ': case '\t': case '\n': case '\f':
+        case ' ': case TAB: case NL: case '\f':
             *(pzScan++) = NUL;
             while (IS_WHITESPACE_CHAR(*pzScan)) pzScan++;
 
@@ -395,7 +395,7 @@ prep_invoke_args(tMacro* pMac)
  *      helpful, but not crucial.  Please contact the author if you think you
  *      might actually want to use this.
 =*/
-tMacro*
+tMacro *
 mFunc_Debug(tTemplate* pT, tMacro* pMac)
 {
     static int dummy = 0;
@@ -438,7 +438,7 @@ mFunc_Debug(tTemplate* pT, tMacro* pMac)
     }
     if (IS_GRAPHIC_CHAR(dummy))
         fprintf(pfTrace, " (%c)", dummy);
-    putc('\n', pfTrace);
+    putc(NL, pfTrace);
     return pMac+1;
 }
 
@@ -466,7 +466,7 @@ build_defs(int defCt, tDefList* pList)
         switch (*(pList->pzExpr)) {
         case ';':
         {
-            char* pz = strchr(pList->pzExpr, '\n');
+            char* pz = strchr(pList->pzExpr, NL);
             if (pz != NULL) {
                 while (IS_WHITESPACE_CHAR(*++pz))  ;
                 pList->pzExpr = pz;
@@ -500,7 +500,7 @@ build_defs(int defCt, tDefList* pList)
             else if (AG_SCM_NUM_P(res)) {
                 pList->de.val.pzText = AGALOC(16, "number buf");
                 snprintf(pList->de.val.pzText, (size_t)16, "%ld",
-                         gh_scm2ulong(res));
+                         AG_SCM_TO_ULONG(res));
             }
             else
                 AGDUPSTR(pList->de.val.pzText, zNil, "empty string");
@@ -742,7 +742,7 @@ mFunc_Invoke(tTemplate* pT, tMacro* pMac)
  *          at load time, too :-)
  */
 tMacro*
-mLoad_Debug(tTemplate* pT, tMacro* pMac, tCC** ppzScan)
+mLoad_Debug(tTemplate* pT, tMacro* pMac, char const ** ppzScan)
 {
     if (OPT_VALUE_TRACE >= TRACE_DEBUG_MESSAGE)
         return mLoad_Unknown(pT, pMac, ppzScan);
@@ -751,7 +751,7 @@ mLoad_Debug(tTemplate* pT, tMacro* pMac, tCC** ppzScan)
 
 
 tMacro*
-mLoad_Define(tTemplate* pT, tMacro* pMac, tCC** ppzScan)
+mLoad_Define(tTemplate* pT, tMacro* pMac, char const ** ppzScan)
 {
     static char const zNameNeeded[] = "DEFINE requires a name";
 
@@ -761,7 +761,7 @@ mLoad_Define(tTemplate* pT, tMacro* pMac, tCC** ppzScan)
     /*
      *  Save the global macro loading mode
      */
-    tpLoadProc* papLP = papLoadProc;
+    tpLoadProc const * papLP = papLoadProc;
     static tpLoadProc apDefineLoad[ FUNC_CT ] = { NULL };
 
     if (pMac->ozText == 0)
