@@ -1,13 +1,13 @@
 
-/*
- *  $Id: f437d17591d8b28b9740a6e8a64947213da9b72f $
+/**
+ * \file tpLoad.c
  *
- * Time-stamp:        "2010-02-24 08:41:33 bkorb"
+ * Time-stamp:        "2011-04-19 17:26:12 bkorb"
  *
  *  This module will load a template and return a template structure.
  *
  * This file is part of AutoGen.
- * AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ * AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,54 +27,65 @@ static tTlibMark magicMark = TEMPLATE_MAGIC_MARKER;
 
 /* = = = START-STATIC-FORWARD = = = */
 static ag_bool
-canReadFile( tCC* pzFName );
+read_okay(char const * pzFName);
 
 static size_t
-countMacros( tCC* pz );
+cnt_macros(char const * pz);
 
 static void
-loadMacros( tTemplate* pT,
-            tCC*       pzF,
-            tCC*       pzN,
-            tCC*       pzData );
+load_macs(tTemplate * pT, char const * pzF, char const * pzN,
+          char const * pzData);
+
+static tTemplate *
+digest_pseudo_macro(tmap_info_t * minfo, char * real_file);
+
+static void
+wrap_up_depends(void);
 /* = = = END-STATIC-FORWARD = = = */
 
-
-LOCAL tTemplate*
-findTemplate( tCC* pzTemplName )
+/**
+ * Return the template structure matching the name passed in.
+ */
+LOCAL tTemplate *
+findTemplate(char const * pzTemplName)
 {
-    tTemplate* pT = pNamedTplList;
+    tTemplate * pT = pNamedTplList;
     while (pT != NULL) {
-        if (streqvcmp( pzTemplName, pT->pzTplName ) == 0)
+        if (streqvcmp(pzTemplName, pT->pzTplName) == 0)
             break;
         pT = (tTemplate*)(void*)(pT->pNext);
     }
     return pT;
 }
 
+/**
+ * the name is a regular file with read access
+ */
 static ag_bool
-canReadFile( tCC* pzFName )
+read_okay(char const * pzFName)
 {
     struct stat stbf;
-    if (stat( pzFName, &stbf ) != 0)
+    if (stat(pzFName, &stbf) != 0)
         return AG_FALSE;
-    if (! S_ISREG( stbf.st_mode ))
+    if (! S_ISREG(stbf.st_mode))
         return AG_FALSE;
-    return (access( pzFName, R_OK) == 0) ? AG_TRUE : AG_FALSE;
+    return (access(pzFName, R_OK) == 0) ? AG_TRUE : AG_FALSE;
 }
 
 
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *
+/**
  *  Starting with the current directory, search the directory
  *  list trying to find the base template file name.
  */
 LOCAL tSuccess
-findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
+findFile(char const * pzFName,
+         char * pzFullName,
+         char const * const * papSuffixList,
+         char const * pzReferrer)
 {
-    char*   pzRoot;
-    char*   pzSfx;
-    void*   deallocAddr = NULL;
+    char * pzRoot;
+    char * pzSfx;
+    void * deallocAddr = NULL;
 
     tSuccess res = SUCCESS;
 
@@ -83,11 +94,11 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
      *  We will not mess with embedded ones.
      */
     if (*pzFName == '$') {
-        if (! optionMakePath( pzFullName, (int)AG_PATH_MAX, pzFName,
-                              autogenOptions.pzProgPath ))
+        if (! optionMakePath(pzFullName, (int)AG_PATH_MAX, pzFName,
+                             autogenOptions.pzProgPath))
             return FAILURE;
 
-        AGDUPSTR( pzFName, pzFullName, "find file name" );
+        AGDUPSTR(pzFName, pzFullName, "find file name");
         deallocAddr = (void*)pzFName;
 
         /*
@@ -102,10 +113,10 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
      *  a suffix for the file name, then append ".tpl".
      *  Check for immediate access once again.
      */
-    pzRoot = strrchr( pzFName, '/' );
+    pzRoot = strrchr(pzFName, '/');
     pzSfx  = (pzRoot != NULL)
-             ? strchr( ++pzRoot, '.' )
-             : strchr( pzFName, '.' );
+             ? strchr(++pzRoot, '.')
+             : strchr(pzFName,  '.');
 
     /*
      *  The referrer is useful only if it includes a directory name.
@@ -133,9 +144,9 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
          *  Search each directory in our directory search list
          *  for the file.  We always force one copy of this option.
          */
-        int     ct     = STACKCT_OPT(  TEMPL_DIRS );
-        tCC**   ppzDir = STACKLST_OPT( TEMPL_DIRS ) + ct - 1;
-        tCC*    pzDir  = curdir;
+        int  ct = STACKCT_OPT(TEMPL_DIRS);
+        char const ** ppzDir = STACKLST_OPT(TEMPL_DIRS) + ct - 1;
+        char const *  pzDir  = curdir;
 
         if (*pzFName == '/')
             ct = 0;
@@ -148,13 +159,13 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
              *  IF one of our template paths starts with '$', then expand it.
              */
             if (*pzDir == '$') {
-                if (! optionMakePath( pzFullName, (int)AG_PATH_MAX, pzDir,
-                                      autogenOptions.pzProgPath )) {
+                if (! optionMakePath(pzFullName, (int)AG_PATH_MAX, pzDir,
+                                     autogenOptions.pzProgPath)) {
                     coerce = (void *)pzDir;
                     strcpy(coerce, curdir);
 
                 } else {
-                    AGDUPSTR( pzDir, pzFullName, "find directory name" );
+                    AGDUPSTR(pzDir, pzFullName, "find directory name");
                     coerce = (void *)ppzDir[1];
                     free(coerce);
                     ppzDir[1] = pzDir; /* save the computed name for later */
@@ -162,16 +173,21 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
             }
 
             if ((*pzFName == '/') || (pzDir == curdir)) {
-                strlcpy(pzFullName, pzFName, AG_PATH_MAX - MAX_SUFFIX_LEN);
-                pzEnd = pzFullName + strlen(pzFullName);
+                size_t nln = strlen(pzFName);
+                if (nln >= AG_PATH_MAX - MAX_SUFFIX_LEN)
+                    break;
+
+                memcpy(pzFullName, pzFName, nln);
+                pzEnd  = pzFullName + nln;
+                *pzEnd = NUL;
             }
             else {
                 pzEnd = pzFullName
-                    + snprintf( pzFullName, AG_PATH_MAX - MAX_SUFFIX_LEN,
-                                zDirFmt, pzDir, pzFName );
+                    + snprintf(pzFullName, AG_PATH_MAX - MAX_SUFFIX_LEN,
+                               zDirFmt, pzDir, pzFName);
             }
 
-            if (canReadFile( pzFullName ))
+            if (read_okay(pzFullName))
                 goto findFileReturn;
 
             /*
@@ -179,12 +195,12 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
              *  THEN try the ones that are okay for this file.
              */
             if ((pzSfx == NULL) && (papSuffixList != NULL)) {
-                tCC** papSL = papSuffixList;
+                char const * const * papSL = papSuffixList;
                 *(pzEnd++) = '.';
 
                 do  {
-                    strcpy( pzEnd, *(papSL++) ); /* must fit */
-                    if (canReadFile( pzFullName ))
+                    strcpy(pzEnd, *(papSL++)); /* must fit */
+                    if (read_okay(pzFullName))
                         goto findFileReturn;
 
                 } while (*papSL != NULL);
@@ -200,9 +216,9 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
              *  through and occasionally "pzDir" will be set to "curdir", but
              *  never again when ct is STACKCT_OPT(TEMPL_DIRS)
              */
-            if (   (pzReferrer != NULL)
-                && (pzDir == curdir)
-                && (ct == STACKCT_OPT(TEMPL_DIRS))) {
+            if (  (pzReferrer != NULL)
+               && (pzDir == curdir)
+               && (ct == STACKCT_OPT(TEMPL_DIRS))) {
 
                 pzDir = pzReferrer;
                 ct++;
@@ -222,23 +238,22 @@ findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
 }
 
 
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *
- *  countMacros
+/**
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  Figure out how many macros there are in the template
  *  so that we can allocate the right number of pointers.
  */
 static size_t
-countMacros( tCC* pz )
+cnt_macros(char const * pz)
 {
     size_t  ct = 2;
     for (;;) {
-        pz = strstr( pz, zStartMac );
+        pz = strstr(pz, zStartMac);
         if (pz == NULL)
             break;
         ct += 2;
-        if (strncmp( pz - endMacLen, zEndMac, endMacLen ) == 0)
+        if (strncmp(pz - endMacLen, zEndMac, endMacLen) == 0)
             ct--;
         pz += startMacLen;
     }
@@ -246,17 +261,14 @@ countMacros( tCC* pz )
 }
 
 
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *
- *  fillTemplate
+/**
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  Load the macro array and file name.
  */
 static void
-loadMacros( tTemplate* pT,
-            tCC*       pzF,
-            tCC*       pzN,
-            tCC*       pzData )
+load_macs(tTemplate * pT, char const * pzF, char const * pzN,
+          char const * pzData)
 {
     tMacro* pMac   = pT->aMacros;
 
@@ -264,10 +276,10 @@ loadMacros( tTemplate* pT,
         char*   pzText = (char*)(pMac + pT->macroCt);
         size_t  len;
 
-        pT->pzTplFile = strdup(pzF);
+        AGDUPSTR(pT->pzTplFile, pzF, "templ file");
 
-        len = strlen( pzN ) + 1;
-        memcpy( (void*)pzText, (void*)pzN, len );
+        len = strlen(pzN) + 1;
+        memcpy((void*)pzText, (void*)pzN, len);
         pT->pzTplName   = pzText;
         pzText         += len;
         pT->pzTemplText = pzText;
@@ -277,14 +289,14 @@ loadMacros( tTemplate* pT,
     pCurTemplate = pT;
 
     {
-        tMacro* pMacEnd = parseTemplate( pMac, &pzData );
+        tMacro* pMacEnd = parseTemplate(pMac, &pzData);
         int     ct;
 
         /*
          *  Make sure all of the input string was scanned.
          */
         if (pzData != NULL)
-            AG_ABEND( "Template parse ended unexpectedly" );
+            AG_ABEND("Template parse ended unexpectedly");
 
         ct = pMacEnd - pMac;
 
@@ -297,7 +309,7 @@ loadMacros( tTemplate* pT,
             void*   data  =
                 (pT->pzTplName == NULL) ? pT->pzTemplText : pT->pzTplName;
             size_t  size  = pT->pNext - (char*)data;
-            memmove( (void*)pMacEnd, data, size );
+            memmove((void*)pMacEnd, data, size);
 
             pT->pzTemplText -= delta;
             pT->pNext       -= delta;
@@ -314,65 +326,54 @@ loadMacros( tTemplate* pT,
      *  the entries are all linked together and
      *  realloc-ing it may cause it to move.
      */
-#if defined( DEBUG_ENABLED )
-    if (HAVE_OPT( SHOW_DEFS )) {
-        tSCC zSum[] = "loaded %d macros from %s\n"
+#if defined(DEBUG_ENABLED)
+    if (HAVE_OPT(SHOW_DEFS)) {
+        static char const zSum[] =
+            "loaded %d macros from %s\n"
             "\tBinary template size:  0x%zX\n\n";
-        fprintf( pfTrace, zSum, pT->macroCt, pzF, pT->descSize );
+        fprintf(pfTrace, zSum, pT->macroCt, pzF, pT->descSize);
     }
 #endif
 }
 
-
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *
- *  Starting with the current directory, search the directory
- *  list trying to find the base template file name.
- */
-LOCAL tTemplate*
-loadTemplate(tCC* pzFileName, tCC * pzReferrer)
+LOCAL void
+append_source_name(char const * pz)
 {
-    static tmap_info_t  mapInfo;
-    static char     zRealFile[ AG_PATH_MAX ];
-    tSCC*           apzSfx[] = { "tpl", "agl", NULL };
+    static char const fmt[] = " \\\n\t%s\n";
+    size_t ln = strlen(pz) + sizeof(fmt) - 2 /* one for '%' and one for 's' */;
+    char * insert_pt;
 
-    tTemplate*      pRes;
-    tMacro*         pSaveMac = pCurMacro;
-
-    /*
-     *  Find the template file somewhere
-     */
-    if (! SUCCESSFUL( findFile(pzFileName, zRealFile, apzSfx, pzReferrer)))
-        AG_ABEND( aprf( zCannot, ENOENT, "map data file", pzFileName,
-                        strerror( ENOENT )));
-
-    /*
-     *  Make sure the specified file is a regular file.
-     *  Make sure the output time stamp is at least as recent.
-     */
-    {
-        struct stat stbf;
-        if (stat( zRealFile, &stbf ) != 0)
-            AG_ABEND( aprf( zCannot, errno, "stat template file", pzFileName,
-                            strerror(errno)));
-
-        if (! S_ISREG( stbf.st_mode ))
-            AG_ABEND( aprf( zCannot, EINVAL, "not regular file", pzFileName,
-                            strerror( EINVAL )));
-
-        if (outTime <= stbf.st_mtime)
-            outTime = stbf.st_mtime + 1;
+    if (source_used + ln >= source_size) {
+        pzSourceList =
+            AGREALOC(pzSourceList, source_size + ln + 2048, "srclist");
+        source_size += ln + 2048;
     }
 
-    text_mmap( zRealFile, PROT_READ|PROT_WRITE, MAP_PRIVATE, &mapInfo );
-    if (TEXT_MMAP_FAILED_ADDR(mapInfo.txt_data))
-        AG_ABEND( aprf( "Could not open template '%s'", zRealFile ));
+    insert_pt = (char *)pzSourceList + source_used;
+    ln = sprintf(insert_pt, fmt, pz);
 
     /*
-     *  Process the leading pseudo-macro.  The template proper
-     *  starts immediately after it.
+     *  See if we've done this file before.
      */
-    pCurMacro = NULL;
+    {
+        char * p = strstr(pzSourceList, insert_pt);
+        if (p == insert_pt) {
+            source_used += ln - 1;
+            p += ln - 1;
+        } else
+            p = insert_pt;
+
+        *p = NUL;
+    }
+}
+
+/**
+ * Process the stuff in the pseudo macro.
+ */
+static tTemplate *
+digest_pseudo_macro(tmap_info_t * minfo, char * real_file)
+{
+    tTemplate * pRes;
 
     /*
      *  Count the number of macros in the template.  Compute
@@ -380,43 +381,109 @@ loadTemplate(tCC* pzFileName, tCC * pzReferrer)
      *  and the size of the template data.  These may get reduced
      *  by comments.
      */
-    {
-        tCC*   pzData   = loadPseudoMacro( (tCC*)mapInfo.txt_data, zRealFile );
-        size_t macroCt  = countMacros( pzData );
-        size_t alocSize = (sizeof( *pRes ) + (macroCt * sizeof( tMacro ))
-            + mapInfo.txt_size - (pzData - (tCC*)mapInfo.txt_data)
-            + strlen(zRealFile) + 0x10) & ~0x0F;
+    char const * pzData =
+        loadPseudoMacro((char const *)minfo->txt_data, real_file);
 
-        pRes = (tTemplate*)AGALOC( alocSize, "main template" );
-        memset( (void*)pRes, 0, alocSize );
+    size_t macroCt  = cnt_macros(pzData);
+    size_t alocSize = (sizeof(*pRes) + (macroCt * sizeof(tMacro))
+                       + minfo->txt_size
+                       - (pzData - (char const *)minfo->txt_data)
+                       + strlen(real_file) + 0x10) & ~0x0F;
 
-        /*
-         *  Initialize the values:
-         */
-        pRes->magic     = magicMark;
-        pRes->descSize  = alocSize;
-        pRes->macroCt   = macroCt;
+    pRes = (tTemplate*)AGALOC(alocSize, "main template");
+    memset((void*)pRes, 0, alocSize);
 
-        strcpy( pRes->zStartMac, zStartMac ); /* must fit */
-        strcpy( pRes->zEndMac, zEndMac );     /* must fit */
-        loadMacros( pRes, zRealFile, "*template file*", pzData );
-    }
+    /*
+     *  Initialize the values:
+     */
+    pRes->magic     = magicMark;
+    pRes->descSize  = alocSize;
+    pRes->macroCt   = macroCt;
+
+    strcpy(pRes->zStartMac, zStartMac); /* must fit */
+    strcpy(pRes->zEndMac, zEndMac);     /* must fit */
+    load_macs(pRes, real_file, "*template file*", pzData);
 
     pRes->pzTplName   -= (long)pRes;
     pRes->pzTemplText -= (long)pRes;
-    pRes = (tTemplate*)AGREALOC( (void*)pRes, pRes->descSize,
-                                 "resize template" );
+    pRes = (tTemplate*)AGREALOC((void*)pRes, pRes->descSize,
+                                "resize template");
     pRes->pzTplName   += (long)pRes;
     pRes->pzTemplText += (long)pRes;
 
-    text_munmap( &mapInfo );
-    pCurMacro    = pSaveMac;
-
     return pRes;
 }
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/**
+ *  Starting with the current directory, search the directory
+ *  list trying to find the base template file name.
+ */
+LOCAL tTemplate *
+loadTemplate(char const * pzFileName, char const * referrer)
+{
+    static tmap_info_t mapInfo;
+    static char        zRealFile[ AG_PATH_MAX ];
+
+    /*
+     *  Find the template file somewhere
+     */
+    {
+        static char const * const apzSfx[] = { "tpl", "agl", NULL };
+        if (! SUCCESSFUL(findFile(pzFileName, zRealFile, apzSfx, referrer))) {
+            errno = ENOENT;
+            AG_CANT("map data file", pzFileName);
+        }
+    }
+
+    /*
+     *  Make sure the specified file is a regular file.
+     *  Make sure the output time stamp is at least as recent.
+     */
+    {
+        struct stat stbf;
+        if (stat(zRealFile, &stbf) != 0)
+            AG_CANT("stat file", pzFileName);
+
+        if (! S_ISREG(stbf.st_mode)) {
+            errno = EINVAL;
+            AG_CANT("not regular file", pzFileName);
+        }
+
+        if (outTime <= stbf.st_mtime)
+            outTime = stbf.st_mtime + 1;
+    }
+
+    text_mmap(zRealFile, PROT_READ|PROT_WRITE, MAP_PRIVATE, &mapInfo);
+    if (TEXT_MMAP_FAILED_ADDR(mapInfo.txt_data))
+        AG_ABEND(aprf("Could not open template '%s'", zRealFile));
+
+    if (pfDepends != NULL)
+        append_source_name(zRealFile);
+
+    /*
+     *  Process the leading pseudo-macro.  The template proper
+     *  starts immediately after it.
+     */
+    {
+        tMacro    * pSaveMac = pCurMacro;
+        tTemplate * pRes;
+        pCurMacro = NULL;
+
+        pRes = digest_pseudo_macro(&mapInfo, zRealFile);
+        pCurMacro = pSaveMac;
+        text_munmap(&mapInfo);
+
+        return pRes;
+    }
+}
+
+/**
+ * Deallocate anything we allocated related to a template,
+ * including the pointer passed in.
+ */
 LOCAL void
-unloadTemplate( tTemplate* pT )
+unloadTemplate(tTemplate* pT)
 {
     tMacro* pMac = pT->aMacros;
     int ct = pT->macroCt;
@@ -426,29 +493,96 @@ unloadTemplate( tTemplate* pT )
         unsigned int ix = pMac->funcCode;
 
         /*
-         *  "select" functions get remapped, depending on the alias used
-         *  for the selection.  See the "teFuncType" enumeration in functions.h.
+         * "select" functions get remapped, depending on the alias used for
+         * the selection.  See the "teFuncType" enumeration in functions.h.
          */
         if (ix >= FUNC_CT)
             ix = FTYP_SELECT;
 
         proc = apUnloadProc[ ix ];
         if (proc != NULL)
-            (*proc)( pMac );
+            (*proc)(pMac);
 
         pMac++;
     }
 
-    AGFREE( (void*)(pT->pzTplFile) );
-    AGFREE( pT );
+    AGFREE((void*)(pT->pzTplFile));
+    AGFREE(pT);
 }
 
+/**
+ *  Finish off the dependency file
+ */
+static void
+wrap_up_depends(void)
+{
+    static mode_t const fil_mode =
+        S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+
+    static char const fmt[] =
+        "\n\n%1$s_SList =%2$s\n"
+        "\n%3$s : $(%1$s_SList)\n"
+        "\n$(%1$s_TList) : %3$s\n"
+        "\t@:\n";
+
+    fprintf(pfDepends, fmt, pz_targ_base, pzSourceList, pzDepTarget);
+    fclose(pfDepends);
+    pfDepends = NULL;
+
+    /*
+     * "startTime" is engineered to be 1 second earlier than the mod time
+     * of any of the output files.
+     */
+    {
+        struct utimbuf tbuf = {
+            .actime  = time(NULL),
+            .modtime = startTime
+        };
+
+        utime(pzDepFile, &tbuf);
+
+        /*
+         * If the target is not the dependency file, then ensure that the
+         * file exists and set its time to the same time.  Ignore all errors.
+         */
+        if (strcmp(pzDepFile, pzDepTarget) != 0) {
+            if (access(pzDepTarget, R_OK) != 0)
+                close( open(pzDepTarget, O_CREAT, fil_mode));
+
+            utime(pzDepTarget, &tbuf);
+        }
+    }
+
+    chmod(pzDepFile, fil_mode);
+
+    /*
+     * Trim off the temporary suffix and rename the dependency file into
+     * place.  We used a mkstemp name in case autogen failed.
+     */
+    {
+        char * pze = strrchr(pzDepFile, '-');
+        char * pzn;
+
+        *pze = NUL;
+        AGDUPSTR(pzn, pzDepFile, "dep file");
+        *pze = '-';
+        rename(pzDepFile, pzn);
+        AGFREE(pzn);
+    }
+}
+
+/**
+ *  This gets called when all is well at the end.
+ */
 LOCAL void
-cleanup( tTemplate* pTF )
+cleanup(tTemplate* pTF)
 {
     if (HAVE_OPT(USED_DEFINES))
         print_used_defines();
 
+    if (pfDepends != NULL)
+        wrap_up_depends();
+
     optionFree(&autogenOptions);
 
     for (;;) {
@@ -456,15 +590,12 @@ cleanup( tTemplate* pTF )
         if (pT == NULL)
             break;
         pNamedTplList = (tTemplate*)(void*)(pT->pNext);
-        unloadTemplate( pT );
+        unloadTemplate(pT);
     }
 
-    AGFREE( forInfo.fi_data );
-    unloadTemplate( pTF );
+    AGFREE(forInfo.fi_data);
+    unloadTemplate(pTF);
     unloadDefs();
-    ag_scmStrings_deinit();
-
-    manageAllocatedData( NULL );
 }
 
 /*
