@@ -1,13 +1,13 @@
-/*
- *  agTempl.c
- *  $Id: 3ba9563a24d217b0f13ef2d4466a3dae57a57e7e $
+
+/**
+ *  \file tpProcess.c
  *
  *  Parse and process the template data descriptions
  *
- * Time-stamp:        "2009-11-01 14:09:38 bkorb"
+ * Time-stamp:        "2011-04-20 14:22:43 bkorb"
  *
  * This file is part of AutoGen.
- * AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ * AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -26,27 +26,24 @@
 static tFpStack fpRoot = { 0, NULL, NULL, NULL };
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static void
-doStdoutTemplate( tTemplate* pTF );
+trace_macro(tTemplate * pT, tMacro * pMac);
 
 static void
-openOutFile( tOutSpec* pOutSpec, tFpStack* pStk );
+do_stdout_tpl(tTemplate * pTF);
+
+static void
+open_output(tOutSpec * spec);
 /* = = = END-STATIC-FORWARD = = = */
 
-/*
+/**
  *  Generate all the text within a block.  The caller must
  *  know the exact bounds of the block.  "pEnd" actually
  *  must point to the first entry that is *not* to be emitted.
  */
 LOCAL void
-generateBlock( tTemplate*   pT,
-               tMacro*      pMac,
-               tMacro*      pEnd )
+generateBlock(tTemplate * pT, tMacro * pMac, tMacro * pEnd)
 {
-    tSCC zFmt[] = "%-10s (%2X) in %s at line %d\n";
-    int  fc;
-
     /*
      *  Set up the processing context for this block of macros.
      *  It is used by the Guile callback routines and the exception
@@ -55,44 +52,62 @@ generateBlock( tTemplate*   pT,
     pCurTemplate = pT;
 
     while ((pMac != NULL) && (pMac < pEnd)) {
-        fc = pMac->funcCode;
+        teFuncType fc = pMac->funcCode;
         if (fc >= FUNC_CT)
             fc = FTYP_BOGUS;
 
-        if (OPT_VALUE_TRACE >= TRACE_EVERYTHING) {
-
-            fprintf( pfTrace, zFmt, apzFuncNames[ fc ], pMac->funcCode,
-                     pT->pzTplFile, pMac->lineNo );
-            if (pMac->ozText > 0) {
-                int   ct;
-                char* pz = pT->pzTemplText + pMac->ozText;
-                fputs( "  ", pfTrace );
-                for (ct=0; ct < 32; ct++) {
-                    char ch = *(pz++);
-                    if (ch == NUL)
-                        break;
-                    if (ch == '\n')
-                        break;
-                    putc( ch, pfTrace );
-                }
-                putc( '\n', pfTrace );
-            }
-        }
+        if (OPT_VALUE_TRACE >= TRACE_EVERYTHING)
+            trace_macro(pT, pMac);
 
         pCurMacro = pMac;
-        pMac = (*(apHdlrProc[ fc ]))( pT, pMac );
-        ag_scmStrings_free();
+        pMac = (*(apHdlrProc[ fc ]))(pT, pMac);
+        ag_scribble_free();
     }
 }
 
-
+/**
+ *  Print out information about the invocation of a macro
+ */
 static void
-doStdoutTemplate( tTemplate* pTF )
+trace_macro(tTemplate * pT, tMacro * pMac)
 {
-    tSCC zNoSfx[] = "* NONE *";
-    tSCC zBadR[]  = "%sBogus return from setjmp\n";
-    SCM  res;
-    tCC* pzRes;
+    static char const zFmt[] = "%-10s (%2X) in %s at line %d\n";
+
+    teFuncType fc = pMac->funcCode;
+    if (fc >= FUNC_CT)
+        fc = FTYP_BOGUS;
+
+    fprintf(pfTrace, zFmt, apzFuncNames[fc], pMac->funcCode,
+            pT->pzTplFile, pMac->lineNo);
+
+    if (pMac->ozText > 0) {
+        int   ct;
+        char* pz = pT->pzTemplText + pMac->ozText;
+        fputs("  ", pfTrace);
+        for (ct=0; ct < 32; ct++) {
+            char ch = *(pz++);
+            if (ch == NUL)
+                break;
+            if (ch == NL)
+                break;
+            putc(ch, pfTrace);
+        }
+        putc(NL, pfTrace);
+    }
+}
+
+/**
+ *  The template output goes to stdout.  Perhaps because output
+ *  is for a CGI script.  In any case, this case must be handled
+ *  specially.
+ */
+static void
+do_stdout_tpl(tTemplate * pTF)
+{
+    static char const zNoSfx[] = "* NONE *";
+    static char const zBadR[]  = "%sBogus return from setjmp\n";
+    char   const *    pzRes;
+    SCM    res;
 
     pzLastScheme = NULL; /* We cannot be in Scheme processing */
 
@@ -102,17 +117,17 @@ doStdoutTemplate( tTemplate* pTF )
 
     case PROBLEM:
         if (*pzOopsPrefix != NUL) {
-            fprintf( stdout, "%soutput was abandoned\n", pzOopsPrefix );
+            fprintf(stdout, "%soutput was abandoned\n", pzOopsPrefix);
             pzOopsPrefix = zNil;
         }
-        fclose( stdout );
+        fclose(stdout);
         return;
 
     default:
         fprintf(stdout, zBadR, pzOopsPrefix);
 
     case FAILURE:
-        exit( EXIT_FAILURE );
+        exit(EXIT_FAILURE);
     }
 
     pzCurSfx      = zNoSfx;
@@ -122,7 +137,7 @@ doStdoutTemplate( tTemplate* pTF )
     fpRoot.pzOutName = "stdout";
     fpRoot.flags  = FPF_NOUNLINK | FPF_STATIC_NM;
     if (OPT_VALUE_TRACE >= TRACE_EVERYTHING)
-        fputs( "Starting stdout template\n", pfTrace );
+        fputs("Starting stdout template\n", pfTrace);
 
     /*
      *  IF there is a CGI prefix for error messages,
@@ -130,31 +145,34 @@ doStdoutTemplate( tTemplate* pTF )
      *  the output will be clean for any error messages we have to emit.
      */
     if (*pzOopsPrefix == NUL)
-        generateBlock( pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt );
+        generateBlock(pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt);
 
     else {
-        tSCC zCont[]  = "content-type:";
-        (void)ag_scm_out_push_new( SCM_UNDEFINED );
+        static char const zCont[]  = "content-type:";
+        (void)ag_scm_out_push_new(SCM_UNDEFINED);
 
-        generateBlock( pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt );
+        generateBlock(pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt);
 
         /*
          *  Read back in the spooled output.  Make sure it starts with
          *  a content-type: prefix.  If not, we supply our own HTML prefix.
          */
-        res   = ag_scm_out_pop( SCM_BOOL_T );
-        pzRes = AG_SCM_CHARS( res );
+        res   = ag_scm_out_pop(SCM_BOOL_T);
+        pzRes = AG_SCM_CHARS(res);
 
-        if (strneqvcmp( pzRes, zCont, (int)sizeof(zCont) - 1) != 0)
-            fputs( "Content-Type: text/html\n\n", stdout );
+        if (strneqvcmp(pzRes, zCont, (int)sizeof(zCont) - 1) != 0)
+            fputs("Content-Type: text/html\n\n", stdout);
 
-        fwrite( pzRes, AG_SCM_STRLEN(res), (size_t)1, stdout );
+        fwrite(pzRes, AG_SCM_STRLEN(res), (size_t)1, stdout);
     }
 
-    fclose( stdout );
+    fclose(stdout);
 }
 
-
+/**
+ * pop the current output spec structure.  Deallocate it and the
+ * file name, too, if necessary.
+ */
 LOCAL tOutSpec *
 nextOutSpec(tOutSpec * pOS)
 {
@@ -169,7 +187,7 @@ nextOutSpec(tOutSpec * pOS)
 
 
 LOCAL void
-processTemplate( tTemplate* pTF )
+processTemplate(tTemplate* pTF)
 {
     forInfo.fi_depth = 0;
 
@@ -179,7 +197,7 @@ processTemplate( tTemplate* pTF )
      *  With output going to stdout, we don't try to remove output on errors.
      */
     if (pOutSpecList == NULL) {
-        doStdoutTemplate( pTF );
+        do_stdout_tpl(pTF);
         return;
     }
 
@@ -197,24 +215,28 @@ processTemplate( tTemplate* pTF )
         /*
          *  HOW was that we got here?
          */
-        switch (setjmp( fileAbort )) {
+        switch (setjmp(fileAbort)) {
         case SUCCESS:
+            if (OPT_VALUE_TRACE >= TRACE_EVERYTHING) {
+                fprintf(pfTrace, "Starting %s template\n", pOS->zSuffix);
+                fflush(pfTrace);
+            }
             /*
              *  Set the output file name buffer.
-             *  It may get switched inside openOutFile.
+             *  It may get switched inside open_output.
              */
-            openOutFile( pOS, &fpRoot );
-
-            pzCurSfx = pOS->zSuffix;
-            currDefCtx = rootDefCtx;
-            if (OPT_VALUE_TRACE >= TRACE_EVERYTHING) {
-                fprintf( pfTrace, "Starting %s template\n", pzCurSfx );
-                fflush( pfTrace );
-            }
-            generateBlock( pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt );
+            open_output(pOS);
+            memcpy(&fpRoot, pCurFp, sizeof(fpRoot));
+            AGFREE(pCurFp);
+            pCurFp         = &fpRoot;
+            pzCurSfx       = pOS->zSuffix;
+            currDefCtx     = rootDefCtx;
+            pCurFp->flags &= ~FPF_FREE;
+            pCurFp->pPrev  = NULL;
+            generateBlock(pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt);
 
             do  {
-                closeOutput( AG_FALSE );  /* keep output */
+                closeOutput(AG_FALSE);  /* keep output */
             } while (pCurFp->pPrev != NULL);
             break;
 
@@ -229,7 +251,7 @@ processTemplate( tTemplate* pTF )
             break;
 
         default:
-            fprintf( pfTrace, "%sBogus return from setjmp\n", pzOopsPrefix );
+            fprintf(pfTrace, "%sBogus return from setjmp\n", pzOopsPrefix);
             pzOopsPrefix = zNil;
             /* FALLTHROUGH */
 
@@ -256,12 +278,15 @@ processTemplate( tTemplate* pTF )
 
 
 LOCAL void
-closeOutput( ag_bool purge )
+closeOutput(ag_bool purge)
 {
     if ((pCurFp->flags & FPF_NOCHMOD) == 0)
-        removeWriteAccess( fileno( pCurFp->pFile ));
+        make_readonly(fileno(pCurFp->pFile));
 
-    fclose( pCurFp->pFile );
+    if (OPT_VALUE_TRACE > TRACE_DEBUG_MESSAGE)
+        fprintf(pfTrace, "closeOutput '%s'\n", pCurFp->pzOutName);
+
+    fclose(pCurFp->pFile);
 
     /*
      *  Only stdout and /dev/null are marked, "NOUNLINK"
@@ -272,15 +297,22 @@ closeOutput( ag_bool purge )
          *  file, then get rid of the output.
          */
         if (purge || ((pCurFp->flags & FPF_UNLINK) != 0))
-            unlink( pCurFp->pzOutName );
+            unlink(pCurFp->pzOutName);
 
         else {
             struct utimbuf tbuf;
 
-            tbuf.actime  = time( NULL );
+            tbuf.actime  = time(NULL);
             tbuf.modtime = outTime;
 
-            utime( pCurFp->pzOutName, &tbuf );
+            /*
+             *  The putative start time is one second earlier than the
+             *  earliest output file time, regardless of when that is.
+             */
+            if (outTime <= startTime)
+                startTime = outTime - 1;
+
+            utime(pCurFp->pzOutName, &tbuf);
         }
     }
 
@@ -288,7 +320,7 @@ closeOutput( ag_bool purge )
      *  Do not deallocate statically allocated names
      */
     if ((pCurFp->flags & FPF_STATIC_NM) == 0)
-        AGFREE( (void*)pCurFp->pzOutName );
+        AGFREE((void*)pCurFp->pzOutName);
 
     /*
      *  Do not deallocate the root entry.  It is not allocated!!
@@ -296,81 +328,28 @@ closeOutput( ag_bool purge )
     if ((pCurFp->flags & FPF_FREE) != 0) {
         tFpStack* p = pCurFp;
         pCurFp = p->pPrev;
-        AGFREE( (void*)p );
+        AGFREE((void*)p);
     }
 }
 
 
+/**
+ *  Figure out what to use as the base name of the output file.
+ *  If an argument is not provided, we use the base name of
+ *  the definitions file.
+ */
 static void
-openOutFile( tOutSpec* pOutSpec, tFpStack* pStk )
+open_output(tOutSpec * spec)
 {
-    tCC*  pzDefFile;
+    static char const write_mode[] = "w" FOPEN_BINARY_FLAG "+";
 
-    /*
-     *  Reset the post-output control flags.
-     */
-    pStk->flags = 0;
+    char const * out_file = NULL;
 
-    /*
-     *  Figure out what to use as the base name of the output file.
-     *  If an argument is not provided, we use the base name of
-     *  the definitions file.
-     */
-    pzDefFile = OPT_ARG( BASE_NAME );
-
-    /*
-     *  Remove any suffixes in the last file name
-     */
-    {
-        char   z[ AG_PATH_MAX ];
-        tCC*   pS = strrchr( pzDefFile, '/' );
-        char*  pE;
-
-        // coverity[dereference] -- invalid analysis -- "pS" not dereferenced.
-        if (pS++ == NULL)
-            pS = pzDefFile;
-
-        /*
-         *  We allow users to specify a suffix with '-' and '_', but when
-         *  stripping a suffix from the "base name", we do not recognize 'em.
-         */
-        pE = strchr( pS, '.' );
-        if (pE != NULL) {
-            size_t len = (unsigned)(pE - pS);
-            if (len >= sizeof(z))
-                AG_ABEND( "--base-name name is too long" );
-
-            memcpy(z, pS, len);
-            z[ pE - pS ] = NUL;
-            pS = z;
-        }
-
-        /*
-         *  Now formulate the output file name in the buffer
-         *  provided as the input argument.
-         */
-        pStk->pzOutName = aprf( pOutSpec->pzFileFmt, pS,
-                                pOutSpec->zSuffix );
-        if (pStk->pzOutName == NULL)
-            AG_ABEND( aprf( "Cannot format file name:  ``%s''",
-                            pOutSpec->pzFileFmt ));
-    }
-
-    pCurFp = pStk;
-
-    if (strcmp( pOutSpec->zSuffix, "null" ) == 0) {
-      null_open:
-        /*
-         *  Make the output a no-op, but perform the operations.
-         */
-        AGFREE( (void*)pStk->pzOutName );
-        pStk->pzOutName = zDevNull;
-        pStk->flags    |= FPF_STATIC_NM | FPF_NOUNLINK | FPF_NOCHMOD;
-        pStk->pFile     = fopen( zDevNull, "w" FOPEN_BINARY_FLAG "+" );
-        if (pStk->pFile != NULL)
-            return;
-
-        goto openError;
+    if (strcmp(spec->zSuffix, "null") == 0) {
+        static int const flags = FPF_NOUNLINK | FPF_NOCHMOD | FPF_TEMPFILE;
+    null_open:
+        open_output_file(zDevNull, sizeof(zDevNull)-1, write_mode, flags);
+        return;
     }
 
     /*
@@ -378,24 +357,56 @@ openOutFile( tOutSpec* pOutSpec, tFpStack* pStk )
      *  we will redirect the output to /dev/null and
      *  perform all the work.  There may be side effects.
      */
-    if (HAVE_OPT( SKIP_SUFFIX )) {
-        int     ct  = STACKCT_OPT(  SKIP_SUFFIX );
-        tCC**   ppz = STACKLST_OPT( SKIP_SUFFIX );
+    if (HAVE_OPT(SKIP_SUFFIX)) {
+        int     ct  = STACKCT_OPT(SKIP_SUFFIX);
+        const char ** ppz = STACKLST_OPT(SKIP_SUFFIX);
 
         while (--ct >= 0) {
-            if (strcmp( pOutSpec->zSuffix, *ppz++ ) == 0)
+            if (strcmp(spec->zSuffix, *ppz++) == 0)
                 goto null_open;
         }
     }
 
-    unlink( pStk->pzOutName );
-    pStk->pFile = fopen( pStk->pzOutName, "w+" FOPEN_BINARY_FLAG );
+    /*
+     *  Remove any suffixes in the last file name
+     */
+    {
+        static char const bad_fmt[] =
+            "Cannot format file name:  \"%s\", %s, %s";
 
-    if (pStk->pFile == NULL) {
-    openError:
-        AG_ABEND( aprf( zCannot, errno, "create", pStk->pzOutName,
-                        strerror( errno )));
+        char const * def_file = OPT_ARG(BASE_NAME);
+        char   z[AG_PATH_MAX];
+        const char * pst = strrchr(def_file, '/');
+        char * end;
+
+        pst = (pst == NULL) ? def_file : (pst + 1);
+
+        /*
+         *  We allow users to specify a suffix with '-' and '_', but when
+         *  stripping a suffix from the "base name", we do not recognize 'em.
+         */
+        end = strchr(pst, '.');
+        if (end != NULL) {
+            size_t len = (unsigned)(end - pst);
+            if (len >= sizeof(z))
+                AG_ABEND("--base-name name is too long");
+
+            memcpy(z, pst, len);
+            z[ end - pst ] = NUL;
+            pst = z;
+        }
+
+        /*
+         *  Now formulate the output file name in the buffer
+         *  provided as the input argument.
+         */
+        out_file = aprf(spec->pzFileFmt, pst, spec->zSuffix);
+        if (out_file == NULL)
+            AG_ABEND(aprf(bad_fmt, spec->pzFileFmt, pst, spec->zSuffix));
     }
+
+    open_output_file(out_file, strlen(out_file), write_mode, 0);
+    free((void *)out_file);
 }
 /*
  * Local Variables:
