@@ -1,7 +1,7 @@
 /**
  * \file configfile.c
  *
- *  Time-stamp:      "2011-03-06 13:41:17 bkorb"
+ *  Time-stamp:      "2011-05-02 12:03:39 bkorb"
  *
  *  configuration/rc/ini file handling.
  *
@@ -26,9 +26,6 @@
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
-static void
-set_usage_flags(tOptions * opts, char const * flg_txt);
-
 /* = = = START-STATIC-FORWARD = = = */
 static void
 file_preset(tOptions * opts, char const * fname, int dir);
@@ -662,7 +659,7 @@ program_directive(tOptions * pOpts, char * pzText)
     size_t name_len = strlen(pOpts->pzProgName);
 
     memcpy(ttl, ttlfmt, sizeof(ttlfmt) - 1);
-    strcpy(ttl + sizeof(ttlfmt) - 1, zCfgProg);
+    memcpy(ttl + sizeof(ttlfmt) - 1, zCfgProg, ttl_len - (sizeof(ttlfmt) - 1));
 
     do  {
         while (IS_WHITESPACE_CHAR(*++pzText))  ;
@@ -1026,15 +1023,15 @@ internalFileLoad(tOptions* pOpts)
 
         if (S_ISDIR(StatBuf.st_mode)) {
             size_t len = strlen(zFileName);
-            char* pz;
+            size_t nln = strlen(pOpts->pzRcName) + 1;
+            char * pz  = zFileName + len;
 
-            if (len + 1 + strlen(pOpts->pzRcName) >= sizeof(zFileName))
+            if (len + 1 + nln >= sizeof(zFileName))
                 continue;
 
-            pz = zFileName + len;
             if (pz[-1] != DIRCH)
                 *(pz++) = DIRCH;
-            strcpy(pz, pOpts->pzRcName);
+            memcpy(pz, pOpts->pzRcName, nln);
         }
 
         file_preset(pOpts, zFileName, inc);
